@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.10.2.3 2004/01/13 15:39:41 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.17 2003/10/29 00:08:01 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,9 +56,9 @@ EventContext::~EventContext ()
 {
 }
 
-nsresult EventContext::Init (EphyWrapper *wrapper)
+nsresult EventContext::Init (EphyBrowser *browser)
 {
-	mWrapper = wrapper;
+	mBrowser = browser;
 	mDOMDocument = nsnull;
 
 	return NS_OK;
@@ -145,14 +145,10 @@ nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
 nsresult EventContext::ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
 {
 	nsresult rv;
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *base;
-	base = doc->GetBaseURI ();
-	if (!base) return NS_ERROR_FAILURE;
-#elif MOZILLA_SNAPSHOT > 11
+#if MOZILLA_SNAPSHOT > 11
 	nsIURI *base;
 	base = doc->GetBaseURL ();
-	if (!base) return NS_ERROR_FAILURE;
+	if (base == NULL) return NS_ERROR_FAILURE;
 #elif MOZILLA_SNAPSHOT > 9 
 	nsCOMPtr<nsIURI> base;
 	rv = doc->GetBaseURL (getter_AddRefs(base));
@@ -169,14 +165,10 @@ nsresult EventContext::ResolveBaseURL (nsIDocument *doc, const nsAString &relurl
 nsresult EventContext::ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
 {
 	nsresult rv;
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *uri;
-	uri = doc->GetDocumentURI ();
-	if (!uri) return NS_ERROR_FAILURE;
-#elif MOZILLA_SNAPSHOT > 11
+#if MOZILLA_SNAPSHOT > 11
 	nsIURI *uri;
 	uri = doc->GetDocumentURL ();
-	if (!uri) return NS_ERROR_FAILURE;
+	if (uri == NULL) return NS_ERROR_FAILURE;
 #else
 	nsCOMPtr<nsIURI> uri;
 	rv = doc->GetDocumentURL(getter_AddRefs(uri));
@@ -777,9 +769,9 @@ nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, EphyEmbedEven
 nsresult EventContext::IsPageFramed (nsIDOMNode *node, PRBool *Framed)
 {
 	nsresult result;
-	
+
 	nsCOMPtr<nsIDOMDocument> mainDocument;
-	result = mWrapper->GetMainDOMDocument (getter_AddRefs(mainDocument));
+	result = mBrowser->GetDocument (getter_AddRefs(mainDocument));
 	if (NS_FAILED(result) || !mainDocument) return NS_ERROR_FAILURE;
 	
 	nsCOMPtr<nsIDOMDocument> nodeDocument;
