@@ -17,15 +17,14 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #if MOZILLA_SNAPSHOT < 12
 
-#include <gtk/gtk.h>
-#include <libgnome/gnome-exec.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-url.h>
+#include <gtk/gtkmessagedialog.h>
+#include <glib/gi18n.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include <nsString.h>
 #include <nsXPIDLString.h>
@@ -35,7 +34,6 @@
 #include <nsIWindowWatcher.h>
 #include <nsIServiceManager.h>
 
-#include "ephy-prefs.h"
 #include "eel-gconf-extensions.h"
 
 #include "ExternalProtocolService.h"
@@ -47,13 +45,10 @@ NS_IMPL_ISUPPORTS1(GExternalProtocolService, nsIExternalProtocolService)
 
 GExternalProtocolService::GExternalProtocolService()
 {
-  NS_INIT_ISUPPORTS();
-  /* member initializers and constructor code */
 }
 
 GExternalProtocolService::~GExternalProtocolService()
 {
-  /* destructor code */
 }
 
 /* boolean externalProtocolHandlerExists (in string aProtocolScheme); */
@@ -106,7 +101,7 @@ NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
 	char *result = eel_gconf_get_string(key.get());
 	if (result)
 	{
-		gnome_url_show(cSpec.get(), NULL);
+		gnome_vfs_url_show(cSpec.get());
 		g_free (result);
 		return NS_OK;
 	}
@@ -147,7 +142,7 @@ NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
 
 	if (ret == 0)
 	{
-		gnome_url_show(cSpec.get(), NULL);		
+		gnome_vfs_url_show(cSpec.get());		
 		return NS_OK;
 	}
 	else
@@ -158,4 +153,4 @@ NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
 	return NS_OK;
 }
 
-#endif /* MOZILLA_SNAPSHOT < 12 */
+#endif
