@@ -24,6 +24,7 @@
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkdnd.h>
+#include <gtk/gtkhbox.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkmenu.h>
@@ -31,7 +32,7 @@
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtktoolitem.h>
 #include <gtk/gtkseparatortoolitem.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <string.h>
 
 static void egg_editable_toolbar_class_init	(EggEditableToolbarClass *klass);
@@ -59,8 +60,8 @@ enum
 
 enum
 {
-	ACTION_REQUEST,
-	LAST_SIGNAL
+  ACTION_REQUEST,
+  LAST_SIGNAL
 };
 
 static guint egg_editable_toolbar_signals[LAST_SIGNAL] = { 0 };
@@ -73,6 +74,10 @@ struct EggEditableToolbarPrivate
   EggToolbarsModel *model;
   gboolean edit_mode;
   GtkWidget *selected_toolbar;
+  GtkWidget *fixed_toolbar;
+
+  gboolean pending;
+  GtkToolbar *target_toolbar;
   GtkWidget *dragged_item;
 };
 
@@ -110,7 +115,7 @@ get_toolbar_position (EggEditableToolbar *etoolbar, GtkWidget *toolbar)
   int result;
 
   l = gtk_container_get_children (GTK_CONTAINER (etoolbar));
-  result = g_list_index (l, toolbar);
+  result = g_list_index (l, toolbar->parent);
   g_list_free (l);
 
   return result;
@@ -130,8 +135,8 @@ get_n_toolbars (EggEditableToolbar *etoolbar)
 }
 
 static GtkWidget *
-get_toolbar_nth (EggEditableToolbar *etoolbar,
-		 int                 position)
+get_dock_nth (EggEditableToolbar *etoolbar,
+	      int                 position)
 {
   GList *l;
   GtkWidget *result;
@@ -143,6 +148,23 @@ get_toolbar_nth (EggEditableToolbar *etoolbar,
   return result;
 }
 
+static GtkWidget *
+get_toolbar_nth (EggEditableToolbar *etoolbar,
+		 int                 position)
+{
+  GList *l;
+  GtkWidget *dock;
+  GtkWidget *result;
+
+  dock = get_dock_nth (etoolbar, position);
+
+  l = gtk_container_get_children (GTK_CONTAINER (dock));
+  result = GTK_WIDGET (l->data);
+  g_list_free (l);
+
+  return result;
+}
+
 static GtkAction *
 find_action (EggEditableToolbar *t,
 	     const char         *name)
@@ -238,7 +260,7 @@ set_drag_cursor (GtkWidget *widget)
 
       pixbuf = gdk_pixbuf_new_from_file (CURSOR_DIR "/art/hand-open.png", NULL);
       cursor = gdk_cursor_new_from_pixbuf (gdk_display_get_default (),
-					   pixbuf, 0, 0);
+					   pixbuf, 12, 12);
       gdk_window_set_cursor (widget->window, cursor);
       gdk_cursor_unref (cursor);
       g_object_unref (pixbuf);
@@ -290,7 +312,8 @@ set_item_drag_source (GtkWidget *item,
 static GtkWidget *
 create_item_from_action (EggEditableToolbar *t,
 			 const char *action_name,
-			 gboolean is_separator)
+			 gboolean is_separator,
+			 GtkAction **ret_action)
 {
   GtkWidget *item;
   GtkAction *action;
@@ -304,8 +327,16 @@ create_item_from_action (EggEditableToolbar *t,
     {
       g_signal_emit (G_OBJECT (t), egg_editable_toolbar_signals[ACTION_REQUEST],
 		     0, action_name);
+
       action = find_action (t, action_name);
-      item = gtk_action_create_tool_item (action);
+      if (action)
+        {
+          item = gtk_action_create_tool_item (action);
+        }
+      else
+        {
+          return NULL;
+        }  
     }
 
   gtk_widget_show (item);
@@ -327,6 +358,11 @@ create_item_from_action (EggEditableToolbar *t,
       gtk_tool_item_set_use_drag_window (GTK_TOOL_ITEM (item), TRUE);
     }
 
+  if (ret_action)
+    {
+      *ret_action = action;
+    }
+
   return item;
 }
 
@@ -334,15 +370,15 @@ static GtkWidget *
 create_item (EggEditableToolbar *t,
 	     EggToolbarsModel   *model,
 	     int                 toolbar_position,
-	     int                 position)
+	     int                 position,
+             GtkAction         **ret_action)
 {
   const char *action_name;
   gboolean is_separator;
 
   action_name = egg_toolbars_model_item_nth
-		(model, toolbar_position, position,
-		 &is_separator);
-  return create_item_from_action (t, action_name, is_separator);
+		(model, toolbar_position, position, &is_separator);
+  return create_item_from_action (t, action_name, is_separator, ret_action);
 }
 
 static gboolean
@@ -351,22 +387,6 @@ data_is_separator (const char *data)
   return strcmp (data, "separator") == 0;
 }
 
-static void
-set_status_pending (GdkDragContext *context,
-                    GdkDragAction   suggested_action)
-{
-  g_object_set_data (G_OBJECT (context),
-                     "gtk-calendar-status-pending",
-                     GINT_TO_POINTER (suggested_action));
-}
-                                                                                                                             
-static GdkDragAction
-get_status_pending (GdkDragContext *context)
-{
-  return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (context),
-                                             "gtk-calendar-status-pending"));
-}
-
 static void
 drag_data_received_cb (GtkWidget          *widget,
 		       GdkDragContext     *context,
@@ -377,46 +397,62 @@ drag_data_received_cb (GtkWidget          *widget,
 		       guint               time,
 		       EggEditableToolbar *etoolbar)
 {
-  GdkDragAction suggested_action;
-  int pos, toolbar_pos;
+  char *type;
+  char *id;
 
-  suggested_action = get_status_pending (context);
+  GdkAtom target;
+	  
+  target = gtk_drag_dest_find_target (widget, context, NULL);
+  type = egg_toolbars_model_get_item_type (etoolbar->priv->model, target);
+  id = egg_toolbars_model_get_item_id (etoolbar->priv->model, type,
+				       selection_data->data);
 
-  if (suggested_action)
+  /* This function can be called for two reasons
+   *
+   *  (1) drag_motion() needs an item to pass to
+   *      gtk_toolbar_set_drop_highlight_item(). We can
+   *      recognize this case by etoolbar->priv->pending being TRUE
+   *      We should just create an item and return.
+   *
+   *  (2) The drag has finished, and drag_drop() wants us to
+   *      actually add a new item to the toolbar.
+   */
+
+  if (etoolbar->priv->pending)
     {
-      set_status_pending (context, 0);
+      etoolbar->priv->pending = FALSE;
       etoolbar->priv->dragged_item =
-        create_item_from_action (etoolbar, selection_data->data,
-				 data_is_separator (selection_data->data));
-      gdk_drag_status (context, suggested_action, time);
-      return;
+        create_item_from_action (etoolbar, id, data_is_separator (id), NULL);
+    }
+  else
+    {
+      int pos, toolbar_pos;
+
+      pos = gtk_toolbar_get_drop_index (GTK_TOOLBAR (widget), x, y);
+      toolbar_pos = get_toolbar_position (etoolbar, widget);
+
+      if (data_is_separator (selection_data->data))
+	{
+	  egg_toolbars_model_add_separator (etoolbar->priv->model,
+					    toolbar_pos, pos);
+	}
+      else
+	{
+	  type = egg_toolbars_model_get_item_type (etoolbar->priv->model,
+						   target);
+	  id = egg_toolbars_model_get_item_id (etoolbar->priv->model, type,
+					       selection_data->data);
+
+	  egg_toolbars_model_add_item (etoolbar->priv->model,
+				       toolbar_pos, pos, id, type);
+	}
+      
+      gtk_drag_finish (context, TRUE, context->action == GDK_ACTION_MOVE,
+		       time);
     }
 
-   pos = gtk_toolbar_get_drop_index (GTK_TOOLBAR (widget), x, y);
-   toolbar_pos = get_toolbar_position (etoolbar, widget);
-
-   if (data_is_separator (selection_data->data))
-     {
-       egg_toolbars_model_add_separator (etoolbar->priv->model,
-	  				 toolbar_pos, pos);
-     }
-     else
-     {
-       GdkAtom target;
-       char *type;
-       char *id;
-
-       target = gtk_drag_dest_find_target (widget, context, NULL);
-       type = egg_toolbars_model_get_item_type (etoolbar->priv->model, target);
-       id = egg_toolbars_model_get_item_id (etoolbar->priv->model, type,
-					    selection_data->data);
-       egg_toolbars_model_add_item (etoolbar->priv->model,
-			            toolbar_pos, pos, id, type);
-       g_free (type);
-       g_free (id);
-    }
-
-    gtk_drag_finish (context, TRUE, context->action == GDK_ACTION_MOVE, time);
+  g_free (type);
+  g_free (id);
 }
 
 static void
@@ -475,10 +511,11 @@ static void
 free_dragged_item (EggEditableToolbar *etoolbar)
 {
   if (etoolbar->priv->dragged_item)
-  {	
-    gtk_widget_destroy (etoolbar->priv->dragged_item);
-    etoolbar->priv->dragged_item = NULL;
-  }
+    {
+      gtk_widget_destroy (etoolbar->priv->dragged_item);
+      g_object_unref (etoolbar->priv->dragged_item);
+      etoolbar->priv->dragged_item = NULL;
+    }
 }
 
 static gboolean
@@ -491,8 +528,6 @@ toolbar_drag_drop_cb (GtkWidget          *widget,
 {
   GdkAtom target;
 
-  free_dragged_item (etoolbar);
-  
   target = gtk_drag_dest_find_target (widget, context, NULL);
   if (target != GDK_NONE)
     {
@@ -502,6 +537,8 @@ toolbar_drag_drop_cb (GtkWidget          *widget,
       return TRUE;
     }
   
+  free_dragged_item (etoolbar);
+  
   return FALSE;
 }
 
@@ -514,22 +551,44 @@ toolbar_drag_motion_cb (GtkWidget          *widget,
 		        EggEditableToolbar *etoolbar)
 {
   GdkAtom target;
-  GtkRequisition req;
+  int index;
+  GtkToolbar *toolbar = GTK_TOOLBAR (widget);
+  GtkToolItem *item;
 
   target = gtk_drag_dest_find_target (widget, context, NULL);
   if (target == GDK_NONE)
     {
       gdk_drag_status (context, 0, time);
-    }
-  else
-    {
-      set_status_pending (context, context->suggested_action);
-      gtk_drag_get_data (widget, context, target, time);
+      return FALSE;
     }
 
-  gtk_widget_size_request (etoolbar->priv->dragged_item, &req);
-  gtk_toolbar_highlight_drop_location (GTK_TOOLBAR (widget), x, y,
-				       req.width, req.height);
+  if (etoolbar->priv->target_toolbar != toolbar)
+    {
+      if (etoolbar->priv->target_toolbar)
+	gtk_toolbar_set_drop_highlight_item
+		(etoolbar->priv->target_toolbar, NULL, 0);
+      
+      free_dragged_item (etoolbar);
+      etoolbar->priv->pending = TRUE;
+
+      etoolbar->priv->target_toolbar = toolbar;
+
+      /* The handler will make sure the item is created */
+      gtk_drag_get_data (widget, context, target, time);
+
+      g_assert (etoolbar->priv->dragged_item);
+      g_assert (!etoolbar->priv->pending);
+
+      g_object_ref (etoolbar->priv->dragged_item);
+      gtk_object_sink (GTK_OBJECT (etoolbar->priv->dragged_item));
+    }
+
+  item = GTK_TOOL_ITEM (etoolbar->priv->dragged_item);
+
+  index = gtk_toolbar_get_drop_index (toolbar, x, y);
+  gtk_toolbar_set_drop_highlight_item (toolbar, item, index);
+
+  gdk_drag_status (context, context->suggested_action, time);
 
   return TRUE;
 }
@@ -540,19 +599,37 @@ toolbar_drag_leave_cb (GtkToolbar         *toolbar,
 		       guint               time,
 		       EggEditableToolbar *etoolbar)
 {
-  free_dragged_item (etoolbar);
-  gtk_toolbar_unhighlight_drop_location (toolbar);
+  /* This is a workaround for bug 125557. Sometimes
+   * we seemingly enter another toolbar *before* leaving
+   * the current one.
+   *
+   * In that case etoolbar->priv->target_toolbar will
+   * have been set to something else and the highlighting
+   * will already have been turned off
+   */
+  
+  if (etoolbar->priv->target_toolbar == toolbar)
+    {
+      gtk_toolbar_set_drop_highlight_item (toolbar, NULL, 0);
+
+      etoolbar->priv->target_toolbar = NULL;
+      free_dragged_item (etoolbar);
+    }
 }
 
 static GtkWidget *
-create_toolbar (EggEditableToolbar *t)
+create_dock (EggEditableToolbar *t)
 {
-  GtkWidget *toolbar;
+  GtkWidget *toolbar, *hbox;
+
+  hbox = gtk_hbox_new (0, FALSE);
+  gtk_widget_show (hbox);
 
   toolbar = gtk_toolbar_new ();
   gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), TRUE);
-
   gtk_widget_show (toolbar);
+  gtk_box_pack_start (GTK_BOX (hbox), toolbar, TRUE, TRUE, 0);
+
   gtk_drag_dest_set (toolbar, 0,
 		     dest_drag_types, n_dest_drag_types,
 		     GDK_ACTION_MOVE | GDK_ACTION_COPY);
@@ -569,7 +646,19 @@ create_toolbar (EggEditableToolbar *t)
   g_signal_connect (toolbar, "popup_context_menu",
 		    G_CALLBACK (popup_toolbar_context_menu_cb), t);
 
-  return toolbar;
+  return hbox;
+}
+
+static void
+set_fixed_style (EggEditableToolbar *t, GtkToolbarStyle style)
+{
+  gtk_toolbar_set_style (GTK_TOOLBAR (t->priv->fixed_toolbar), style);
+}
+
+static void
+unset_fixed_style (EggEditableToolbar *t)
+{
+  gtk_toolbar_unset_style (GTK_TOOLBAR (t->priv->fixed_toolbar));
 }
 
 static void
@@ -586,25 +675,69 @@ toolbar_changed_cb (EggToolbarsModel   *model,
   if (flags & EGG_TB_MODEL_ICONS_ONLY)
   {
     gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+    if (position == 0)
+      {
+        set_fixed_style (t, GTK_TOOLBAR_ICONS);
+      }
   }
   else
   {
     gtk_toolbar_unset_style (GTK_TOOLBAR (toolbar));
+    if (position == 0)
+      {
+        unset_fixed_style (t);
+      }
   }
 }
 
+static void
+unparent_fixed (EggEditableToolbar *t)
+{
+  GtkWidget *toolbar, *dock;
+
+  toolbar = t->priv->fixed_toolbar;
+  dock = get_dock_nth (t, 0);
+
+  if (dock && toolbar->parent != NULL)
+    {
+      gtk_container_remove (GTK_CONTAINER (dock), toolbar);
+    }
+}
+
+static void
+update_fixed (EggEditableToolbar *t)
+{
+  GtkWidget *toolbar, *dock;
+
+  toolbar = t->priv->fixed_toolbar;
+  dock = get_dock_nth (t, 0);
+
+  if (dock && toolbar && toolbar->parent == NULL)
+    {
+      gtk_box_pack_end (GTK_BOX (dock), toolbar, FALSE, TRUE, 0);
+
+      gtk_widget_show_all (toolbar);
+    }
+}
+
 static void
 toolbar_added_cb (EggToolbarsModel   *model,
 	          int                 position,
 	          EggEditableToolbar *t)
 {
-  GtkWidget *toolbar;
+  GtkWidget *dock;
 
-  toolbar = create_toolbar (t);
-  gtk_widget_set_size_request (toolbar, -1, MIN_TOOLBAR_HEIGHT);
-  gtk_box_pack_start (GTK_BOX (t), toolbar, FALSE, FALSE, 0);
+  dock = create_dock (t);
 
-  gtk_box_reorder_child (GTK_BOX (t), toolbar, position);
+  gtk_widget_set_size_request (dock, -1, MIN_TOOLBAR_HEIGHT);
+
+  gtk_box_pack_start (GTK_BOX (t), dock, TRUE, TRUE, 0);
+
+  gtk_box_reorder_child (GTK_BOX (t), dock, position);
+
+  gtk_widget_show_all (dock);
+  
+  update_fixed (t);
 }
 
 static void
@@ -614,8 +747,10 @@ toolbar_removed_cb (EggToolbarsModel   *model,
 {
   GtkWidget *toolbar;
 
-  toolbar = get_toolbar_nth (t, position);
+  toolbar = get_dock_nth (t, position);
   gtk_widget_destroy (toolbar);
+
+  update_fixed (t);
 }
 
 static void
@@ -626,12 +761,19 @@ item_added_cb (EggToolbarsModel   *model,
 {
   GtkWidget *toolbar;
   GtkWidget *item;
+  GtkAction *action;
 
   toolbar = get_toolbar_nth (t, toolbar_position);
   gtk_widget_set_size_request (toolbar, -1, -1);
-  item = create_item (t, model, toolbar_position, position);
+  item = create_item (t, model, toolbar_position, position, &action);
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
 		      GTK_TOOL_ITEM (item), position);
+
+  /* FIXME Hack to make tooltip work from gtk */
+  if (action)
+    {
+      g_object_notify (G_OBJECT (action), "tooltip");
+    }
 }
 
 static void
@@ -650,10 +792,10 @@ item_removed_cb (EggToolbarsModel   *model,
   gtk_container_remove (GTK_CONTAINER (toolbar), item);
 
   if (egg_toolbars_model_n_items (model, toolbar_position) == 0)
-  {
-    gtk_widget_set_size_request (toolbar, -1, MIN_TOOLBAR_HEIGHT);
-    egg_toolbars_model_remove_toolbar (model, toolbar_position);
-  }
+    {
+      gtk_widget_set_size_request (toolbar, -1, MIN_TOOLBAR_HEIGHT);
+      egg_toolbars_model_remove_toolbar (model, toolbar_position);
+    }
 }
 
 static void
@@ -661,6 +803,9 @@ toolbars_clean (EggEditableToolbar *t)
 {
   GList *children, *l;
 
+  unset_fixed_style (t);
+  unparent_fixed (t);
+
   children = gtk_container_get_children (GTK_CONTAINER (t));
 
   for (l = children; l != NULL; l = l->next)
@@ -683,26 +828,46 @@ egg_editable_toolbar_construct (EggEditableToolbar *t)
 
   for (i = 0; i < n_toolbars; i++)
     {
-      GtkWidget *toolbar;
+      GtkWidget *toolbar, *dock;
       EggTbModelFlags flags;
 
-      toolbar = create_toolbar (t);
-      gtk_box_pack_start (GTK_BOX (t), toolbar, FALSE, FALSE, 0);
+      dock = create_dock (t);
+      gtk_box_pack_start (GTK_BOX (t), dock, TRUE, TRUE, 0);
+      toolbar = get_toolbar_nth (t, i);
 
       flags = egg_toolbars_model_get_flags (model, i);
       if (flags & EGG_TB_MODEL_ICONS_ONLY)
         {
           gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+          if (i == 0)
+            {
+	      set_fixed_style (t, GTK_TOOLBAR_ICONS);
+            }
         }
 
       n_items = egg_toolbars_model_n_items (model, i);
       for (l = 0; l < n_items; l++)
         {
           GtkWidget *item;
+          GtkAction *action;
 
-          item = create_item (t, model, i, l);
-	  gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
-			      GTK_TOOL_ITEM (item), l);
+          item = create_item (t, model, i, l, &action);
+          if (item)
+            {
+	      gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
+			          GTK_TOOL_ITEM (item), l);
+              /* FIXME Hack to make tooltip work from gtk */
+              if (action)
+                {
+                  g_object_notify (G_OBJECT (action), "tooltip");
+                }
+            }
+          else
+            {
+              egg_toolbars_model_remove_item (model, i, l);
+              l--;
+              n_items--;
+            }
         }
 
       if (n_items == 0)
@@ -710,6 +875,8 @@ egg_editable_toolbar_construct (EggEditableToolbar *t)
             gtk_widget_set_size_request (toolbar, -1, MIN_TOOLBAR_HEIGHT);
         }
     }
+
+    update_fixed (t);
 }
 
 static void
@@ -846,6 +1013,11 @@ egg_editable_toolbar_finalize (GObject *object)
   g_return_if_fail (object != NULL);
   g_return_if_fail (IS_EGG_EDITABLE_TOOLBAR (object));
 
+  if (t->priv->fixed_toolbar)
+    {
+      g_object_unref (t->priv->fixed_toolbar);
+    }
+
   g_free (t->priv);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -933,7 +1105,7 @@ egg_editable_toolbar_show (EggEditableToolbar *etoolbar,
       toolbar_name = egg_toolbars_model_toolbar_nth (model, i);
       if (strcmp (toolbar_name, name) == 0)
       {
-        gtk_widget_show (get_toolbar_nth (etoolbar, i));
+        gtk_widget_show (get_dock_nth (etoolbar, i));
       }
     }
 }
@@ -955,11 +1127,33 @@ egg_editable_toolbar_hide (EggEditableToolbar *etoolbar,
       toolbar_name = egg_toolbars_model_toolbar_nth (model, i);
       if (strcmp (toolbar_name, name) == 0)
       {
-        gtk_widget_hide (get_toolbar_nth (etoolbar, i));
+        gtk_widget_hide (get_dock_nth (etoolbar, i));
       }
     }
 }
 
+GtkToolbar *
+egg_editable_toolbar_set_fixed (EggEditableToolbar *etoolbar,
+				GtkWidget          *fixed)
+{
+  GtkWidget *fixed_item;
+
+  etoolbar->priv->fixed_toolbar = gtk_toolbar_new ();
+  gtk_toolbar_set_show_arrow (GTK_TOOLBAR (etoolbar->priv->fixed_toolbar), FALSE);
+  g_object_ref (etoolbar->priv->fixed_toolbar);
+  gtk_object_sink (GTK_OBJECT (etoolbar->priv->fixed_toolbar));
+
+  fixed_item = GTK_WIDGET (gtk_tool_item_new ());
+  gtk_toolbar_insert (GTK_TOOLBAR (etoolbar->priv->fixed_toolbar),
+                      GTK_TOOL_ITEM (fixed_item), 0);
+
+  gtk_container_add (GTK_CONTAINER (fixed_item), fixed);
+
+  update_fixed (etoolbar);
+
+  return GTK_TOOLBAR (etoolbar->priv->fixed_toolbar);
+}
+
 void
 egg_editable_toolbar_set_drag_dest (EggEditableToolbar   *etoolbar,
 				    const GtkTargetEntry *targets,
