@@ -20,9 +20,11 @@
 #define EGG_EDITABLE_TOOLBAR_H
 
 #include "egg-toolbars-model.h"
-#include "egg-menu-merge.h"
 
+#include <gtk/gtkuimanager.h>
+#include <gtk/gtkselection.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtktoolbar.h>
 
 G_BEGIN_DECLS
 
@@ -42,6 +44,8 @@ typedef struct EggEditableToolbarPrivate EggEditableToolbarPrivate;
 struct EggEditableToolbar
 {
   GtkVBox parent_object;
+
+  /*< private >*/
   EggEditableToolbarPrivate *priv;
 };
 
@@ -54,7 +58,7 @@ struct EggEditableToolbarClass
 };
 
 GType               egg_editable_toolbar_get_type        (void);
-GtkWidget	   *egg_editable_toolbar_new		 (EggMenuMerge         *merge,
+GtkWidget	   *egg_editable_toolbar_new		 (GtkUIManager         *merge,
 							  EggToolbarsModel     *model);
 void		    egg_editable_toolbar_set_edit_mode	 (EggEditableToolbar   *etoolbar,
 							  gboolean              mode);
@@ -66,10 +70,13 @@ void		    egg_editable_toolbar_set_drag_dest   (EggEditableToolbar   *etoolbar,
 							  const GtkTargetEntry *targets,
 							  gint                  n_targets,
 							  const char           *toolbar_name);
+GtkToolbar	   *egg_editable_toolbar_set_fixed       (EggEditableToolbar   *etoolbar,
+							  GtkWidget            *fixed);
+
 
 /* Private Functions */
 
-GtkWidget 	   *_egg_editable_toolbar_new_separator_image ();
+GtkWidget 	   *_egg_editable_toolbar_new_separator_image (void);
 
 G_END_DECLS
 
