@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.h,v 1.3 2003/05/25 19:39:07 mpeseng Exp $
+ *  $Id: ephy-file-helpers.h,v 1.5 2003/11/08 12:56:43 chpe Exp $
  */
 
 #ifndef EPHY_FILE_HELPERS_H
@@ -34,10 +34,6 @@ void        ephy_file_helpers_init       (void);
 
 void        ephy_file_helpers_shutdown   (void);
 
-void	    ephy_file_launch_application (const char *command_string,
-					  const char *parameter,
-					  gboolean use_terminal);
-
 char       *ephy_file_tmp_filename	 (const char *base,
 					  const char *extension);
 
@@ -47,8 +43,8 @@ GSList     *ephy_file_find               (const char *path,
 	                                  const char *fname,
 	                                  gint maxdepth);
 
-gboolean    ephy_file_save_xml           (const char *xml_file,
-					  xmlDocPtr doc);
+gboolean    ephy_file_switch_temp_file   (const char *filename,
+					  const char *filename_temp);
 
 G_END_DECLS
 
