@@ -1,5 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
+ *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,14 +17,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node.h,v 1.12.2.1 2003/12/18 12:48:42 chpe Exp $
+ *  $Id: ephy-node.h,v 1.14 2003/11/19 19:19:52 chpe Exp $
  */
 
-
 #ifndef EPHY_NODE_H
 #define EPHY_NODE_H
 
 #include <libxml/tree.h>
+#include <libxml/xmlwriter.h>
 
 G_BEGIN_DECLS
 
@@ -60,6 +62,10 @@ long        ephy_node_get_id                (EphyNode *node);
 void        ephy_node_ref                   (EphyNode *node);
 void        ephy_node_unref                 (EphyNode *node);
 
+/* locking */
+void        ephy_node_freeze                (EphyNode *node);
+void        ephy_node_thaw                  (EphyNode *node);
+
 /* signals */
 int         ephy_node_signal_connect_object (EphyNode *node,
 					     EphyNodeSignalType type,
@@ -96,8 +102,8 @@ char       *ephy_node_get_property_time     (EphyNode *node,
 					     guint property_id);
 
 /* xml storage */
-void          ephy_node_save_to_xml         (EphyNode *node,
-					     xmlNodePtr parent_xml_node);
+int           ephy_node_write_to_xml	    (EphyNode *node,
+					     xmlTextWriterPtr writer);
 EphyNode     *ephy_node_new_from_xml        (EphyNodeDb *db,
 					     xmlNodePtr xml_node);
 
