@@ -15,18 +15,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.26 2003/08/24 14:49:48 chpe Exp $
+ *  $Id: ephy-node-view.c,v 1.34 2003/10/05 23:55:13 marco Exp $
  */
 
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtktreeviewcolumn.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcellrendererpixbuf.h>
+#include <gtk/gtktreemodelfilter.h>
 #include <gtk/gtkwindow.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
-#include "eggtreemodelfilter.h"
 #include "ephy-node-view.h"
 #include "ephy-tree-model-sort.h"
 #include "eggtreemultidnd.h"
@@ -46,6 +46,9 @@ static void ephy_node_view_get_property (GObject *object,
 				         GValue *value,
 				         GParamSpec *pspec);
 
+
+#define EPHY_NODE_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NODE_VIEW, EphyNodeViewPrivate))
+
 struct EphyNodeViewPrivate
 {
 	EphyNode *root;
@@ -197,19 +200,14 @@ ephy_node_view_class_init (EphyNodeViewClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+
+	g_type_class_add_private (object_class, sizeof (EphyNodeViewPrivate));
 }
 
 static void
 ephy_node_view_finalize (GObject *object)
 {
-	EphyNodeView *view;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (EPHY_IS_NODE_VIEW (object));
-
-	view = EPHY_NODE_VIEW (object);
-
-	g_return_if_fail (view->priv != NULL);
+	EphyNodeView *view = EPHY_NODE_VIEW (object);
 
 	g_object_unref (G_OBJECT (view->priv->sortmodel));
 	g_object_unref (G_OBJECT (view->priv->filtermodel));
@@ -220,8 +218,6 @@ ephy_node_view_finalize (GObject *object)
 		gtk_target_list_unref (view->priv->source_target_list);
 	}
 
-	g_free (view->priv);
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -236,8 +232,8 @@ get_node_from_path (EphyNodeView *view, GtkTreePath *path)
 	gtk_tree_model_get_iter (view->priv->sortmodel, &iter, path);
 	gtk_tree_model_sort_convert_iter_to_child_iter
 		(GTK_TREE_MODEL_SORT (view->priv->sortmodel), &iter2, &iter);
-	egg_tree_model_filter_convert_iter_to_child_iter
-		(EGG_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
+	gtk_tree_model_filter_convert_iter_to_child_iter
+		(GTK_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
 	node = ephy_tree_model_node_node_from_iter (view->priv->nodemodel, &iter);
 
 	return node;
@@ -358,6 +354,7 @@ drag_motion_cb (GtkWidget *widget,
 		EphyNodeView *view)
 {
 	EphyNode *node;
+	GdkAtom target;
 	GtkTreePath *path;
 	GtkTreeViewDropPosition pos;
 	guint action = 0;
@@ -371,9 +368,10 @@ drag_motion_cb (GtkWidget *widget,
 		get_drag_data (view, context, time);
 	}
 
+	target = gtk_drag_dest_find_target (widget, context, NULL);
 	node = get_node_from_path (view, path);
 
-	if (node)
+	if (target != GDK_NONE && node != NULL)
 	{
 		priority = ephy_node_get_property_int
 				(node, view->priv->priority_prop_id);
@@ -435,17 +433,14 @@ drag_data_received_cb (GtkWidget *widget,
 		       guint32 time,
 		       EphyNodeView *view)
 {
-	GtkTreePath *path;
 	GtkTreeViewDropPosition pos;
-	gboolean on_row;
+
+	/* x and y here are valid only on drop ! */
 
 	if (selection_data->length <= 0 || selection_data->data == NULL)
 	{
 		return;
-	}
-	
-	on_row = gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
-					            x, y, &path, &pos);
+	}	
 
 	if (!view->priv->have_drag_data)
 	{
@@ -459,8 +454,12 @@ drag_data_received_cb (GtkWidget *widget,
 		EphyNode *node;
 		GList *uris;
 		gboolean success = FALSE;
+		GtkTreePath *path;
 
-		g_return_if_fail (on_row && path != NULL);
+		gtk_tree_view_get_dest_row_at_pos
+			(GTK_TREE_VIEW (widget), x, y, &path, &pos);
+
+		g_return_if_fail (path != NULL);
 
 		node = get_node_from_path (view, path);
 
@@ -478,12 +477,13 @@ drag_data_received_cb (GtkWidget *widget,
 		view->priv->drop_occurred = FALSE;
 		free_drag_data (view);
 		gtk_drag_finish (context, success, FALSE, time);
+
+		if (path)
+		{
+			gtk_tree_path_free (path);
+		}
 	}
 
-	if (path)
-	{
-		gtk_tree_path_free (path);
-	}
 
 	/* appease GtkTreeView by preventing its drag_data_receive
 	 * from being called */
@@ -591,8 +591,8 @@ ephy_node_view_row_activated_cb (GtkTreeView *treeview,
 	gtk_tree_model_get_iter (view->priv->sortmodel, &iter, path);
 	gtk_tree_model_sort_convert_iter_to_child_iter
 		(GTK_TREE_MODEL_SORT (view->priv->sortmodel), &iter2, &iter);
-	egg_tree_model_filter_convert_iter_to_child_iter
-		(EGG_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
+	gtk_tree_model_filter_convert_iter_to_child_iter
+		(GTK_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
 
 	node = ephy_tree_model_node_node_from_iter (view->priv->nodemodel, &iter);
 
@@ -739,7 +739,11 @@ did_not_drag (EphyNodeView *view,
                    gtk_tree_selection_path_is_selected (selection, path) &&
                    !button_event_modifies_selection (event))
 		{
-                        gtk_tree_selection_unselect_all (selection);
+			if (gtk_tree_selection_get_mode (selection) == GTK_SELECTION_MULTIPLE)
+			{
+	                        gtk_tree_selection_unselect_all (selection);
+			}
+
                         gtk_tree_selection_select_path (selection, path);
                 }
                                                                                                                               
@@ -957,9 +961,9 @@ ephy_node_view_construct (EphyNodeView *view)
 
 	view->priv->nodemodel = ephy_tree_model_node_new (view->priv->root,
 							  view->priv->filter);
-	view->priv->filtermodel = egg_tree_model_filter_new (GTK_TREE_MODEL (view->priv->nodemodel),
+	view->priv->filtermodel = gtk_tree_model_filter_new (GTK_TREE_MODEL (view->priv->nodemodel),
 							     NULL);
-	egg_tree_model_filter_set_visible_column (EGG_TREE_MODEL_FILTER (view->priv->filtermodel),
+	gtk_tree_model_filter_set_visible_column (GTK_TREE_MODEL_FILTER (view->priv->filtermodel),
 						  EPHY_TREE_MODEL_NODE_COL_VISIBLE);
 	view->priv->sortmodel = ephy_tree_model_sort_new (view->priv->filtermodel);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (view->priv->sortmodel));
@@ -1026,8 +1030,8 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	gtk_tree_model_get_iter (view->priv->sortmodel, &iter, path);
 	gtk_tree_model_sort_convert_iter_to_child_iter
 		(GTK_TREE_MODEL_SORT (view->priv->sortmodel), &iter2, &iter);
-	egg_tree_model_filter_convert_iter_to_child_iter
-		(EGG_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
+	gtk_tree_model_filter_convert_iter_to_child_iter
+		(GTK_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
 	node = ephy_tree_model_node_node_from_iter (view->priv->nodemodel, &iter);
 
 	g_value_init (&value, G_TYPE_STRING);
@@ -1202,9 +1206,6 @@ ephy_node_view_add_column (EphyNodeView *view,
 	int column;
 	int icon_column;
 
-	g_return_val_if_fail (!(flags & EPHY_NODE_VIEW_EDITABLE) ||
-			      view->priv->editable_renderer == NULL, NULL);
-
 	column = ephy_tree_model_node_add_prop_column
 		(view->priv->nodemodel, value_type, prop_id);
 
@@ -1304,7 +1305,8 @@ ephy_node_view_add_column (EphyNodeView *view,
 static void
 ephy_node_view_init (EphyNodeView *view)
 {
-	view->priv = g_new0 (EphyNodeViewPrivate, 1);
+	view->priv = EPHY_NODE_VIEW_GET_PRIVATE (view);
+
 	view->priv->editable_renderer = NULL;
 	view->priv->editing = TRUE;
 	view->priv->searchable_data_column = -1;
@@ -1317,24 +1319,13 @@ static void
 get_selection (GtkTreeModel *model,
 	       GtkTreePath *path,
 	       GtkTreeIter *iter,
-	       void **data)
+	       gpointer *data)
 {
-	GtkTreeModelSort *sortmodel = GTK_TREE_MODEL_SORT (model);
-	EggTreeModelFilter *filtermodel = EGG_TREE_MODEL_FILTER (sortmodel->child_model);
-	EphyTreeModelNode *nodemodel = EPHY_TREE_MODEL_NODE (filtermodel->child_model);
-	GList **list = (GList **) data;
-	GtkTreeIter *iter2 = gtk_tree_iter_copy (iter);
-	GtkTreeIter iter3;
-	GtkTreeIter iter4;
+	GList **list = data[0];
+	EphyNodeView *view = EPHY_NODE_VIEW (data[1]);
 	EphyNode *node;
 
-	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model),
-							&iter3, iter2);
-	egg_tree_model_filter_convert_iter_to_child_iter (filtermodel, &iter4, &iter3);
-
-	node = ephy_tree_model_node_node_from_iter (nodemodel, &iter4);
-
-	gtk_tree_iter_free (iter2);
+	node = get_node_from_path (view, path);
 
 	*list = g_list_prepend (*list, node);
 }
@@ -1344,13 +1335,16 @@ ephy_node_view_get_selection (EphyNodeView *view)
 {
 	GList *list = NULL;
 	GtkTreeSelection *selection;
+	gpointer data[2];
 
 	selection = gtk_tree_view_get_selection	(GTK_TREE_VIEW (view));
 
+	data[0] = &list;
+	data[1] = view;
 	gtk_tree_selection_selected_foreach
 			(selection,
 			 (GtkTreeSelectionForeachFunc) get_selection,
-			 (gpointer) &list);
+			 (gpointer) data);
 
 	return list;
 }
@@ -1375,7 +1369,7 @@ ephy_node_view_remove (EphyNodeView *view)
 	node = g_list_first (list)->data;
 	ephy_tree_model_node_iter_from_node (EPHY_TREE_MODEL_NODE (view->priv->nodemodel),
 					     node, &iter);
-	egg_tree_model_filter_convert_child_iter_to_iter (EGG_TREE_MODEL_FILTER (view->priv->filtermodel),
+	gtk_tree_model_filter_convert_child_iter_to_iter (GTK_TREE_MODEL_FILTER (view->priv->filtermodel),
 							  &iter2, &iter);
 	gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (view->priv->sortmodel),
 							&iter, &iter2);
@@ -1437,7 +1431,7 @@ ephy_node_view_select_node (EphyNodeView *view,
 	g_value_unset (&val);
 	if (visible == FALSE) return;
 
-	egg_tree_model_filter_convert_child_iter_to_iter (EGG_TREE_MODEL_FILTER (view->priv->filtermodel),
+	gtk_tree_model_filter_convert_child_iter_to_iter (GTK_TREE_MODEL_FILTER (view->priv->filtermodel),
 							  &iter2, &iter);
 	gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (view->priv->sortmodel),
 							&iter, &iter2);
