@@ -15,11 +15,11 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: ephy-bookmarks-editor.c,v 1.95 2003/07/27 13:10:56 mpeseng Exp $
+ * $Id: ephy-bookmarks-editor.c,v 1.126 2003/11/17 18:51:42 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <gtk/gtktable.h>
@@ -29,8 +29,12 @@
 #include <gtk/gtkhpaned.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkactiongroup.h>
+#include <gtk/gtktoggleaction.h>
+#include <gtk/gtkuimanager.h>
+#include <gtk/gtktoggleaction.h>
 #include <gdk/gdkkeysyms.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <string.h>
@@ -42,22 +46,20 @@
 #include "ephy-node-view.h"
 #include "ephy-window.h"
 #include "ephy-dnd.h"
-#include "ephy-prefs.h"
 #include "ephy-shell.h"
+#include "ephy-session.h"
 #include "ephy-file-helpers.h"
-#include "egg-action-group.h"
-#include "egg-toggle-action.h"
-#include "egg-menu-merge.h"
-#include "egg-toggle-action.h"
+#include "ephy-file-chooser.h"
 #include "popup-commands.h"
 #include "ephy-state.h"
 #include "window-commands.h"
-#include "ephy-debug.h"
 #include "ephy-gui.h"
 #include "ephy-stock-icons.h"
 #include "ephy-search-entry.h"
 #include "ephy-toolbars-model.h"
 #include "ephy-favicon-cache.h"
+#include "eel-gconf-extensions.h"
+#include "ephy-debug.h"
 
 static GtkTargetEntry topic_drag_dest_types [] =
 {
@@ -94,35 +96,39 @@ static void ephy_bookmarks_editor_get_property (GObject *object,
 						GParamSpec *pspec);
 static void ephy_bookmarks_editor_update_menu  (EphyBookmarksEditor *editor);
 
-static void cmd_open_bookmarks_in_tabs    (EggAction *action,
+static void cmd_open_bookmarks_in_tabs    (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_open_bookmarks_in_browser (EggAction *action,
+static void cmd_open_bookmarks_in_browser (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_show_in_bookmarks_bar     (EggAction *action,
+static void cmd_show_in_bookmarks_bar     (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_delete			  (EggAction *action,
+static void cmd_delete			  (GtkAction *action,
 				           EphyBookmarksEditor *editor);
-static void cmd_bookmark_properties	  (EggAction *action,
+static void cmd_bookmark_properties	  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_bookmarks_import	  (EggAction *action,
+static void cmd_bookmarks_import	  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_add_topic		  (EggAction *action,
+static void cmd_add_topic		  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_rename			  (EggAction *action,
+static void cmd_rename			  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_close			  (EggAction *action,
+static void cmd_close			  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_cut			  (EggAction *action,
+static void cmd_cut			  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_copy			  (EggAction *action,
+static void cmd_copy			  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_paste			  (EggAction *action,
+static void cmd_paste			  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_select_all		  (EggAction *action,
+static void cmd_select_all		  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
-static void cmd_help_contents		  (EggAction *action,
+static void cmd_help_contents		  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
 
+#define EPHY_BOOKMARKS_EDITOR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARKS_EDITOR, EphyBookmarksEditorPrivate))
+
+#define CONF_BOOKMARKS_VIEW_DETAILS "/apps/epiphany/dialogs/bookmarks_view_details"
+
 struct EphyBookmarksEditorPrivate
 {
 	EphyBookmarks *bookmarks;
@@ -130,13 +136,16 @@ struct EphyBookmarksEditorPrivate
 	GtkWidget *key_view;
 	EphyNodeFilter *bookmarks_filter;
 	GtkWidget *search_entry;
-	GtkWidget *menu_dock;
+	GtkWidget *main_vbox;
 	GtkWidget *window;
-	EggMenuMerge *ui_merge;
-	EggActionGroup *action_group;
+	GtkUIManager *ui_merge;
+	GtkActionGroup *action_group;
 	int priority_col;
 	EphyToolbarsModel *tb_model;
 	GHashTable *props_dialogs;
+
+	GtkTreeViewColumn *title_col;
+	GtkTreeViewColumn *address_col;
 };
 
 enum
@@ -147,77 +156,89 @@ enum
 
 static GObjectClass *parent_class = NULL;
 
-static EggActionGroupEntry ephy_bookmark_popup_entries [] = {
+static GtkActionEntry ephy_bookmark_popup_entries [] = {
 	/* Toplevel */
-	{ "File", N_("_File"), NULL, NULL, NULL, NULL, NULL },
-	{ "Edit", N_("_Edit"), NULL, NULL, NULL, NULL, NULL },
-	{ "View", N_("_View"), NULL, NULL, NULL, NULL, NULL },
-	{ "Help", N_("_Help"), NULL, NULL, NULL, NULL, NULL },
-	{ "FakeToplevel", (""), NULL, NULL, NULL, NULL, NULL },
+	{ "File", NULL, N_("_File") },
+	{ "Edit", NULL, N_("_Edit") },
+	{ "View", NULL, N_("_View") },
+	{ "Help", NULL, N_("_Help") },
+	{ "PopupAction", NULL, "" },
 
 	/* File Menu*/
-	{ "NewTopic", N_("_New Topic"), GTK_STOCK_NEW, "<control>N",
+	{ "NewTopic", GTK_STOCK_NEW, N_("_New Topic"), "<control>N",
 	  N_("Create a new topic"), 
-	  G_CALLBACK (cmd_add_topic), NULL },
-	{ "OpenInWindow", N_("_Open in New Window"), GTK_STOCK_OPEN, "<control>O",
+	  G_CALLBACK (cmd_add_topic) },
+	{ "OpenInWindow", GTK_STOCK_OPEN, N_("_Open in New Window"), "<control>O",
 	  N_("Open the selected bookmark in a new window"), 
-	  G_CALLBACK (cmd_open_bookmarks_in_browser), NULL },
-	{ "OpenInTab", N_("Open in New _Tab"), NULL, "<shift><control>O",
+	  G_CALLBACK (cmd_open_bookmarks_in_browser) },
+	{ "OpenInTab", NULL, N_("Open in New _Tab"), "<shift><control>O",
 	  N_("Open the selected bookmark in a new tab"), 
-	  G_CALLBACK (cmd_open_bookmarks_in_tabs), NULL },
-	{ "Rename", N_("_Rename..."), NULL, "F2",
-	  N_("Rename the selected bookmark or topic"), G_CALLBACK (cmd_rename), NULL },
-	{ "Delete", N_("_Delete"), GTK_STOCK_DELETE, NULL,
+	  G_CALLBACK (cmd_open_bookmarks_in_tabs) },
+	{ "Rename", NULL, N_("_Rename..."), "F2",
+	  N_("Rename the selected bookmark or topic"), G_CALLBACK (cmd_rename) },
+	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
 	  N_("Delete the selected bookmark or topic"), 
-	  G_CALLBACK (cmd_delete), NULL },
-	{ "ShowInBookmarksBar", N_("_Show in Bookmarks Bar"), NULL, NULL,
-	  N_("Show the selected bookmark or topic in the bookmarks bar"), 
-	  G_CALLBACK (cmd_show_in_bookmarks_bar), NULL, TOGGLE_ACTION },
-	{ "Properties", N_("_Properties"), GTK_STOCK_PROPERTIES, "<alt>Return",
+	  G_CALLBACK (cmd_delete) },
+	{ "Properties", GTK_STOCK_PROPERTIES, N_("_Properties"), "<alt>Return",
 	  N_("View or modify the properties of the selected bookmark"), 
-	  G_CALLBACK (cmd_bookmark_properties), NULL },
-	{ "Import", N_("_Import Bookmarks..."), NULL, NULL,
+	  G_CALLBACK (cmd_bookmark_properties) },
+	{ "Import", NULL, N_("_Import Bookmarks..."), NULL,
 	  N_("Import bookmarks from another browser or a bookmarks file"), 
-	  G_CALLBACK (cmd_bookmarks_import), NULL },
-	{ "Close", N_("_Close"), GTK_STOCK_CLOSE, "<control>W",
+	  G_CALLBACK (cmd_bookmarks_import) },
+	{ "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
 	  N_("Close the bookmarks window"), 
-	  G_CALLBACK (cmd_close), NULL },
+	  G_CALLBACK (cmd_close) },
 
 	/* Edit Menu */
-	{ "Cut", N_("Cu_t"), GTK_STOCK_CUT, "<control>X",
+	{ "Cut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X",
 	  N_("Cut the selection"), 
-	  G_CALLBACK (cmd_cut), NULL },
-	{ "Copy", N_("_Copy"), GTK_STOCK_COPY, "<control>C",
+	  G_CALLBACK (cmd_cut) },
+	{ "Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
 	  N_("Copy the selection"), 
-	  G_CALLBACK (cmd_copy), NULL },
-	{ "Paste", N_("_Paste"), GTK_STOCK_PASTE, "<control>V",
+	  G_CALLBACK (cmd_copy) },
+	{ "Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
 	  N_("Paste the clipboard"), 
-	  G_CALLBACK (cmd_paste), NULL },
-	{ "SelectAll", N_("Select _All"), NULL, "<control>A",
+	  G_CALLBACK (cmd_paste) },
+	{ "SelectAll", NULL, N_("Select _All"), "<control>A",
 	  N_("Select all bookmarks or text"), 
-	  G_CALLBACK (cmd_select_all), NULL },
+	  G_CALLBACK (cmd_select_all) },
 	
-	/* View Menu */
-	{ "ViewTitle", N_("_Title"), NULL, NULL,
-	  N_("Show only the title column"), 
-	  NULL, NULL, RADIO_ACTION, NULL },
-	{ "ViewLocation", N_("_Address"), NULL, NULL,
-	  N_("Show only the address column"), 
-	  NULL, NULL, RADIO_ACTION, "ViewTitle" },
-	{ "ViewTitleLocation", N_("T_itle and Address"), NULL, NULL,
-	  N_("Show both the title and address columns"), 
-	  NULL, NULL, RADIO_ACTION, "ViewTitle" },	
-
 	/* Help Menu */	
-	{ "HelpContents", N_("_Contents"), GTK_STOCK_HELP, "F1",
+	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1",
 	  N_("Display bookmarks help"), 
-	  G_CALLBACK (cmd_help_contents), NULL },
-	{ "HelpAbout", N_("_About"), GNOME_STOCK_ABOUT, NULL,
+	  G_CALLBACK (cmd_help_contents) },
+	{ "HelpAbout", GNOME_STOCK_ABOUT, N_("_About"), NULL,
 	  N_("Display credits for the web browser creators"),
-	  G_CALLBACK (window_cmd_help_about), NULL },
+	  G_CALLBACK (window_cmd_help_about) },
 };
 static guint ephy_bookmark_popup_n_entries = G_N_ELEMENTS (ephy_bookmark_popup_entries);
 
+static GtkToggleActionEntry ephy_bookmark_popup_toggle_entries [] =
+{
+	/* File Menu */
+	{ "ShowInBookmarksBar", NULL, N_("_Show in Bookmarks Bar"), NULL,
+	  N_("Show the selected bookmark or topic in the bookmarks bar"), 
+	  G_CALLBACK (cmd_show_in_bookmarks_bar), FALSE }
+};
+static guint ephy_bookmark_popup_n_toggle_entries = G_N_ELEMENTS (ephy_bookmark_popup_toggle_entries);
+
+enum
+{
+	VIEW_TITLE,
+	VIEW_TITLE_AND_ADDRESS
+};
+
+static GtkRadioActionEntry ephy_bookmark_radio_entries [] =
+{
+	/* View Menu */
+	{ "ViewTitle", NULL, N_("_Title"), NULL,
+	  N_("Show only the title column"), VIEW_TITLE },
+	{ "ViewTitleAddress", NULL, N_("T_itle and Address"), NULL,
+	  N_("Show both the title and address columns"),
+	  VIEW_TITLE_AND_ADDRESS } 
+};
+static guint ephy_bookmark_n_radio_entries = G_N_ELEMENTS (ephy_bookmark_radio_entries);
+
 static void
 entry_selection_changed_cb (GtkWidget *widget, GParamSpec *pspec, EphyBookmarksEditor *editor)
 {
@@ -249,7 +270,7 @@ add_text_renderer_monitor (EphyBookmarksEditor *editor)
 }
 
 static void
-cmd_add_topic (EggAction *action,
+cmd_add_topic (GtkAction *action,
 	       EphyBookmarksEditor *editor)
 {
 	EphyNode *node;
@@ -262,14 +283,14 @@ cmd_add_topic (EggAction *action,
 }
 
 static void
-cmd_close (EggAction *action,
+cmd_close (GtkAction *action,
 	   EphyBookmarksEditor *editor)
 {
 	gtk_widget_hide (GTK_WIDGET (editor));
 }
 
 static void
-cmd_rename (EggAction *action,
+cmd_rename (GtkAction *action,
 	    EphyBookmarksEditor *editor)
 {
 	if (ephy_node_view_is_target (EPHY_NODE_VIEW (editor->priv->bm_view)))
@@ -292,12 +313,16 @@ get_target_window (EphyBookmarksEditor *editor)
 	}
 	else
 	{
-		return GTK_WIDGET (ephy_shell_get_active_window (ephy_shell));
+		EphySession *session;
+
+		session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+
+		return GTK_WIDGET (ephy_session_get_active_window (session));
 	}
 }
 
 static void
-cmd_show_in_bookmarks_bar (EggAction *action,
+cmd_show_in_bookmarks_bar (GtkAction *action,
 		           EphyBookmarksEditor *editor)
 {
 	EphyNode *node;
@@ -324,7 +349,7 @@ cmd_show_in_bookmarks_bar (EggAction *action,
 
 	node = selection->data;
 	id = ephy_node_get_id (node);
-	state = EGG_TOGGLE_ACTION (action)->active;
+	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 
 	if (state)
 	{
@@ -341,7 +366,7 @@ cmd_show_in_bookmarks_bar (EggAction *action,
 }
 
 static void
-cmd_open_bookmarks_in_tabs (EggAction *action,
+cmd_open_bookmarks_in_tabs (GtkAction *action,
 			    EphyBookmarksEditor *editor)
 {
 	EphyWindow *window;
@@ -367,7 +392,7 @@ cmd_open_bookmarks_in_tabs (EggAction *action,
 }
 
 static void
-cmd_open_bookmarks_in_browser (EggAction *action,
+cmd_open_bookmarks_in_browser (GtkAction *action,
 			       EphyBookmarksEditor *editor)
 {
 	EphyWindow *window;
@@ -386,14 +411,14 @@ cmd_open_bookmarks_in_browser (EggAction *action,
 						EPHY_NODE_BMK_PROP_LOCATION);
 
 		ephy_shell_new_tab (ephy_shell, window, NULL, location,
-				    EPHY_NEW_TAB_IN_NEW_WINDOW);
+				    EPHY_NEW_TAB_OPEN_PAGE | EPHY_NEW_TAB_IN_NEW_WINDOW);
 	}
 
 	g_list_free (selection);
 }
 
 static void
-cmd_delete (EggAction *action,
+cmd_delete (GtkAction *action,
 	    EphyBookmarksEditor *editor)
 {
 	if (ephy_node_view_is_target (EPHY_NODE_VIEW (editor->priv->bm_view)))
@@ -449,27 +474,24 @@ show_properties_dialog (EphyBookmarksEditor *editor, EphyNode *node)
 }
 
 static void
-add_bookmarks_source_menu (GtkWidget *menu,
-			   const char *desc,
-			   const char *dir,
-			   const char *filename)
+add_bookmarks_source (GtkListStore *store,
+		      const char *desc,
+		      const char *dir,
+		      const char *filename,
+		      int max_depth)
 {
+	GtkTreeIter iter;
 	GSList *l;
 	char *path;
 
 	path = g_build_filename (g_get_home_dir (), dir, NULL);
-	l = ephy_file_find  (path, filename, 4);
+	l = ephy_file_find  (path, filename, max_depth);
 	g_free (path);
 
 	if (l)
 	{
-		GtkWidget *item;
-
-		item = gtk_menu_item_new_with_label (desc);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_object_set_data_full (G_OBJECT (item), "bookmarks_file",
-					g_strdup (l->data), g_free);
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, desc, 1, l->data, -1);
 	}
 
 	g_slist_foreach (l, (GFunc) g_free, NULL);
@@ -477,35 +499,83 @@ add_bookmarks_source_menu (GtkWidget *menu,
 }
 
 static void
-import_dialog_response_cb (GtkDialog *dialog, gint response,
-			   GtkWidget *optionmenu)
+import_from_file_response_cb (GtkDialog *dialog, gint response,
+			      EphyBookmarksEditor *editor)
 {
-	if (response == GTK_RESPONSE_OK)
+	char *filename;
+
+	if (response == EPHY_RESPONSE_OPEN)
 	{
-		char *filename;
-		GtkWidget *item, *menu;
-		EphyBookmarks *bookmarks;
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 
-		menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (optionmenu));
-		item = gtk_menu_get_active (GTK_MENU (menu));
-		filename = g_object_get_data (G_OBJECT (item), "bookmarks_file");
+		if (filename != NULL)
+		{
+			ephy_bookmarks_import (editor->priv->bookmarks, filename);			
 
-		bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-		ephy_bookmarks_import (bookmarks, filename);
+			g_free (filename);
+		}
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static void
-cmd_bookmarks_import (EggAction *action,
+import_dialog_response_cb (GtkDialog *dialog, gint response,
+			   EphyBookmarksEditor *editor)
+{
+	if (response == GTK_RESPONSE_OK)
+	{
+		GtkTreeIter iter;
+		char *filename;
+		GtkWidget *combo;
+		GtkTreeModel *model;
+		int active;
+		GValue value = { 0, };
+
+		combo = g_object_get_data (G_OBJECT (dialog), "combo_box");
+		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+		active = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
+		gtk_tree_model_iter_nth_child (model, &iter, NULL, active);
+		gtk_tree_model_get_value (model, &iter, 1, &value);
+		filename = g_strdup (g_value_get_string (&value));
+		g_value_unset (&value);
+
+		if (filename == NULL)
+		{
+			EphyFileChooser *dialog;
+
+			dialog = ephy_file_chooser_new (_("Import bookmarks from file"),
+							GTK_WIDGET (editor),
+							GTK_FILE_CHOOSER_ACTION_OPEN,
+							NULL);
+			/* FIXME: set up some filters perhaps ? */
+			g_signal_connect (dialog, "response",
+					  G_CALLBACK (import_from_file_response_cb), editor);
+
+			gtk_widget_show (GTK_WIDGET (dialog));
+		}
+		else
+		{
+			ephy_bookmarks_import (editor->priv->bookmarks, filename);
+		}
+
+		g_free (filename);
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static void
+cmd_bookmarks_import (GtkAction *action,
 		      EphyBookmarksEditor *editor)
 {
 	GtkWidget *dialog;
 	GtkWidget *label;
 	GtkWidget *vbox;
-	GtkWidget *menu;
-	GtkWidget *option_menu;
+	GtkWidget *combo;
+	GtkCellRenderer *cell;
+	GtkListStore *store;
+	GtkTreeIter iter;
 
 	dialog = gtk_dialog_new_with_buttons (_("Import Bookmarks"),
 					     GTK_WINDOW (editor),
@@ -533,30 +603,42 @@ cmd_bookmarks_import (EggAction *action,
 	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 	gtk_widget_show (label);
 
-	menu = gtk_menu_new ();
-	gtk_widget_show (menu);
+        store = GTK_LIST_STORE (gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING));
+	combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
+	gtk_widget_show (combo); 
+	g_object_set_data (G_OBJECT (dialog), "combo_box", combo);
+	g_object_unref (store);
 
-	add_bookmarks_source_menu (menu, _("Mozilla bookmarks"),
-				   MOZILLA_BOOKMARKS_DIR, "bookmarks.html");
-	add_bookmarks_source_menu (menu, _("Galeon bookmarks"),
-				   GALEON_BOOKMARKS_DIR, "bookmarks.xbel");
-	add_bookmarks_source_menu (menu, _("Konqueror bookmarks"),
-				   KDE_BOOKMARKS_DIR, "bookmarks.xml");
+	cell = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), cell, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), cell,
+                                        "text", 0,
+                                        NULL);
 
-	option_menu = gtk_option_menu_new ();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-	gtk_widget_show (option_menu);
-	gtk_box_pack_start (GTK_BOX (vbox), option_menu, TRUE, TRUE, 0);
+	add_bookmarks_source (store, _("Mozilla bookmarks"),
+			      MOZILLA_BOOKMARKS_DIR, "bookmarks.html", 4);
+	add_bookmarks_source (store, _("Firebird bookmarks"),
+			      FIREBIRD_BOOKMARKS_DIR, "bookmarks.html", 4);
+	add_bookmarks_source (store, _("Galeon bookmarks"),
+			      GALEON_BOOKMARKS_DIR, "bookmarks.xbel", 0);
+	add_bookmarks_source (store, _("Konqueror bookmarks"),
+			      KDE_BOOKMARKS_DIR, "bookmarks.xml", 0);
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("Import from a file"), 1, NULL, -1);
+
+	gtk_box_pack_start (GTK_BOX (vbox), combo, TRUE, TRUE, 0);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (import_dialog_response_cb),
-			  option_menu);
+			  editor);
 
 	gtk_widget_show (dialog);
 }
 
 static void
-cmd_bookmark_properties (EggAction *action,
+cmd_bookmark_properties (GtkAction *action,
 			 EphyBookmarksEditor *editor)
 {
 	GList *selection;
@@ -574,7 +656,7 @@ cmd_bookmark_properties (EggAction *action,
 }
 
 static void
-cmd_cut (EggAction *action,
+cmd_cut (GtkAction *action,
 	 EphyBookmarksEditor *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -586,7 +668,7 @@ cmd_cut (EggAction *action,
 }
 
 static void
-cmd_copy (EggAction *action,
+cmd_copy (GtkAction *action,
 	  EphyBookmarksEditor *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -615,7 +697,7 @@ cmd_copy (EggAction *action,
 }
 
 static void
-cmd_paste (EggAction *action,
+cmd_paste (GtkAction *action,
 	   EphyBookmarksEditor *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -627,7 +709,7 @@ cmd_paste (EggAction *action,
 }
 
 static void
-cmd_select_all (EggAction *action,
+cmd_select_all (GtkAction *action,
 		EphyBookmarksEditor *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -647,7 +729,7 @@ cmd_select_all (EggAction *action,
 }
 
 static void
-cmd_help_contents (EggAction *action,
+cmd_help_contents (GtkAction *action,
 		   EphyBookmarksEditor *editor)
 {
 	ephy_gui_help (GTK_WINDOW (editor), 
@@ -655,6 +737,50 @@ cmd_help_contents (EggAction *action,
 		       "ephy-managing-bookmarks");
 }
 
+static void
+set_columns_visibility (EphyBookmarksEditor *editor, int value)
+{
+	switch (value)
+	{
+		case VIEW_TITLE:
+			gtk_tree_view_column_set_visible (editor->priv->title_col, TRUE);
+			gtk_tree_view_column_set_visible (editor->priv->address_col, FALSE);
+			break;
+		case VIEW_TITLE_AND_ADDRESS:
+			gtk_tree_view_column_set_visible (editor->priv->title_col, TRUE);
+			gtk_tree_view_column_set_visible (editor->priv->address_col, TRUE);
+			break;
+	}
+}
+
+static void
+cmd_view_columns (GtkAction *action,
+		  GtkRadioAction *current,
+		  EphyBookmarksEditor *editor)
+{
+	int value;
+	GSList *svalues = NULL;
+
+	g_return_if_fail (EPHY_IS_BOOKMARKS_EDITOR (editor));
+
+	value = gtk_radio_action_get_current_value (current);
+	set_columns_visibility (editor, value);
+
+	switch (value)
+	{
+		case VIEW_TITLE:
+			svalues = g_slist_append (svalues, (gpointer)"title");
+			break;
+		case VIEW_TITLE_AND_ADDRESS:
+			svalues = g_slist_append (svalues, (gpointer)"title");
+			svalues = g_slist_append (svalues, (gpointer)"address");
+			break;
+	}
+
+	eel_gconf_set_string_list (CONF_BOOKMARKS_VIEW_DETAILS, svalues);
+	g_slist_free (svalues);
+}
+
 GType
 ephy_bookmarks_editor_get_type (void)
 {
@@ -683,10 +809,21 @@ ephy_bookmarks_editor_get_type (void)
 	return ephy_bookmarks_editor_type;
 }
 
+static void
+ephy_bookmarks_editor_show (GtkWidget *widget)
+{
+	EphyBookmarksEditor *editor = EPHY_BOOKMARKS_EDITOR (widget);
+
+	gtk_widget_grab_focus (editor->priv->search_entry);
+
+	GTK_WIDGET_CLASS (parent_class)->show (widget);
+}
+
 static void
 ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -696,32 +833,27 @@ ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass)
 	object_class->set_property = ephy_bookmarks_editor_set_property;
 	object_class->get_property = ephy_bookmarks_editor_get_property;
 
+	widget_class->show = ephy_bookmarks_editor_show;
+
 	g_object_class_install_property (object_class,
 					 PROP_BOOKMARKS,
 					 g_param_spec_object ("bookmarks",
 							      "Bookmarks set",
 							      "Bookmarks set",
-							      EPHY_BOOKMARKS_TYPE,
+							      EPHY_TYPE_BOOKMARKS,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (object_class, sizeof(EphyBookmarksEditorPrivate));
 }
 
 static void
 ephy_bookmarks_editor_finalize (GObject *object)
 {
-	EphyBookmarksEditor *editor;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (EPHY_IS_BOOKMARKS_EDITOR (object));
-
-	editor = EPHY_BOOKMARKS_EDITOR (object);
-
-	g_return_if_fail (editor->priv != NULL);
+	EphyBookmarksEditor *editor = EPHY_BOOKMARKS_EDITOR (object);
 
 	g_object_unref (G_OBJECT (editor->priv->bookmarks_filter));
 
 	g_object_unref (editor->priv->action_group);
-	egg_menu_merge_remove_action_group (editor->priv->ui_merge,
-					    editor->priv->action_group);
 	g_object_unref (editor->priv->ui_merge);
 
 	if (editor->priv->window)
@@ -733,8 +865,6 @@ ephy_bookmarks_editor_finalize (GObject *object)
 
 	g_hash_table_destroy (editor->priv->props_dialogs);
 
-	g_free (editor->priv);
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -744,25 +874,13 @@ ephy_bookmarks_editor_node_activated_cb (GtkWidget *view,
 					 EphyBookmarksEditor *editor)
 {
 	const char *location;
-	EphyWindow *window;
 
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
 	g_return_if_fail (location != NULL);
 
-	window = EPHY_WINDOW (get_target_window (editor));
-	if (window != NULL)
-	{
-		ephy_window_load_url (EPHY_WINDOW (window), location);
-		gtk_window_present (GTK_WINDOW (window));
-	}
-	else
-	{
-		/* We have to create a browser window */
-		ephy_shell_new_tab (ephy_shell, NULL, NULL, location,
-				    EPHY_NEW_TAB_OPEN_PAGE |
-				    EPHY_NEW_TAB_IN_NEW_WINDOW);
-	}
+	ephy_shell_new_tab (ephy_shell, NULL, NULL, location,
+			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
 static void
@@ -777,8 +895,8 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	gboolean bmk_multiple_selection;
 	gboolean cut, copy, paste, select_all;
 	gboolean can_show_in_bookmarks_bar, show_in_bookmarks_bar = FALSE;
-	EggActionGroup *action_group;
-	EggAction *action;
+	GtkActionGroup *action_group;
+	GtkAction *action;
 	GList *selected;
 	GtkWidget *focus_widget;
 
@@ -798,16 +916,13 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	if (GTK_IS_EDITABLE (focus_widget))
 	{
 		gboolean has_selection;
-		gboolean clipboard_contains_text;
 
 		has_selection = gtk_editable_get_selection_bounds
 			(GTK_EDITABLE (focus_widget), NULL, NULL);
-		clipboard_contains_text = gtk_clipboard_wait_is_text_available
-			(gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
 
 		cut = has_selection;
 		copy = has_selection;
-		paste = clipboard_contains_text;
+		paste = TRUE;
 		select_all = TRUE;
 	}
 	else
@@ -883,37 +998,37 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 		 (key_selection && key_focus);
 
 	action_group = editor->priv->action_group;
-	action = egg_action_group_get_action (action_group, "OpenInWindow");
+	action = gtk_action_group_get_action (action_group, "OpenInWindow");
 	g_object_set (action, "sensitive", open_in_window, NULL);
 	g_object_set (action, "label", open_in_window_label, NULL);
-	action = egg_action_group_get_action (action_group, "OpenInTab");
+	action = gtk_action_group_get_action (action_group, "OpenInTab");
 	g_object_set (action, "sensitive", open_in_tab, NULL);
 	g_object_set (action, "label", open_in_tab_label, NULL);
-	action = egg_action_group_get_action (action_group, "Rename");
+	action = gtk_action_group_get_action (action_group, "Rename");
 	g_object_set (action, "sensitive", rename, NULL);
-	action = egg_action_group_get_action (action_group, "Delete");
+	action = gtk_action_group_get_action (action_group, "Delete");
 	g_object_set (action, "sensitive", delete, NULL);
-	action = egg_action_group_get_action (action_group, "Properties");
+	action = gtk_action_group_get_action (action_group, "Properties");
 	g_object_set (action, "sensitive", properties, NULL);
-	action = egg_action_group_get_action (action_group, "Cut");
+	action = gtk_action_group_get_action (action_group, "Cut");
 	g_object_set (action, "sensitive", cut, NULL);
-	action = egg_action_group_get_action (action_group, "Copy");
+	action = gtk_action_group_get_action (action_group, "Copy");
 	g_object_set (action, "sensitive", copy, NULL);
 	g_object_set (action, "label", copy_label, NULL);
-	action = egg_action_group_get_action (action_group, "Paste");
+	action = gtk_action_group_get_action (action_group, "Paste");
 	g_object_set (action, "sensitive", paste, NULL);
-	action = egg_action_group_get_action (action_group, "SelectAll");
+	action = gtk_action_group_get_action (action_group, "SelectAll");
 	g_object_set (action, "sensitive", select_all, NULL);
-	action = egg_action_group_get_action (action_group, "ShowInBookmarksBar");
+	action = gtk_action_group_get_action (action_group, "ShowInBookmarksBar");
 	g_object_set (action, "sensitive", can_show_in_bookmarks_bar, NULL);
 
 	g_signal_handlers_block_by_func
-		(G_OBJECT (EGG_TOGGLE_ACTION (action)),
+		(G_OBJECT (GTK_TOGGLE_ACTION (action)),
 		 G_CALLBACK (cmd_show_in_bookmarks_bar),
 		 editor);
-	egg_toggle_action_set_active (EGG_TOGGLE_ACTION (action), show_in_bookmarks_bar);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), show_in_bookmarks_bar);
 	g_signal_handlers_unblock_by_func
-		(G_OBJECT (EGG_TOGGLE_ACTION (action)),
+		(G_OBJECT (GTK_TOGGLE_ACTION (action)),
 		 G_CALLBACK (cmd_show_in_bookmarks_bar),
 		 editor);
 }
@@ -955,8 +1070,8 @@ ephy_bookmarks_editor_show_popup_cb (GtkWidget *view,
 {
 	GtkWidget *widget;
 
-	widget = egg_menu_merge_get_widget (editor->priv->ui_merge,
-					    "/popups/EphyBookmarkEditorPopup");
+	widget = gtk_ui_manager_get_widget (editor->priv->ui_merge,
+					    "/EphyBookmarkEditorPopup");
 	gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL, 2,
 			gtk_get_current_event_time ());
 }
@@ -1046,8 +1161,8 @@ keyword_node_show_popup_cb (GtkWidget *view, EphyBookmarksEditor *editor)
 {
 	GtkWidget *widget;
 
-	widget = egg_menu_merge_get_widget (editor->priv->ui_merge,
-					   "/popups/EphyBookmarkKeywordPopup");
+	widget = gtk_ui_manager_get_widget (editor->priv->ui_merge,
+					   "/EphyBookmarkKeywordPopup");
 	gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL, 2,
 			gtk_get_current_event_time ());
 }
@@ -1127,9 +1242,9 @@ build_search_box (EphyBookmarksEditor *editor)
 }
 
 static void
-add_widget (EggMenuMerge *merge, GtkWidget *widget, EphyBookmarksEditor *editor)
+add_widget (GtkUIManager *merge, GtkWidget *widget, EphyBookmarksEditor *editor)
 {
-	gtk_box_pack_start (GTK_BOX (editor->priv->menu_dock),
+	gtk_box_pack_start (GTK_BOX (editor->priv->main_vbox),
 			    widget, FALSE, FALSE, 0);
 	gtk_widget_show (widget);
 }
@@ -1185,7 +1300,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	}
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
-	g_value_set_object_take_ownership (value, pixbuf);
+	g_value_take_object (value, pixbuf);
 }
 
 static void
@@ -1207,6 +1322,30 @@ provide_keyword_uri (EphyNode *node, GValue *value, gpointer data)
 	g_free (uri);
 }
 
+static int
+get_details_value (EphyBookmarksEditor *editor)
+{
+	int value;
+	GSList *svalues;
+
+	svalues = eel_gconf_get_string_list (CONF_BOOKMARKS_VIEW_DETAILS);
+
+	if (svalues &&
+	    g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp) &&
+	    g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
+	{
+		value = VIEW_TITLE_AND_ADDRESS;
+	}
+	else
+	{
+		value = VIEW_TITLE;
+	}
+
+	g_slist_free (svalues);
+
+	return value;
+}
+
 static void
 ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 {
@@ -1215,11 +1354,10 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	GtkWidget *bm_view, *key_view;
 	GtkWidget *scrolled_window;
 	EphyNode *node;
-	EggMenuMerge *ui_merge;
-	EggActionGroup *action_group;
-	EggAction *action;
+	GtkUIManager *ui_merge;
+	GtkActionGroup *action_group;
 	GdkPixbuf *icon;
-	int i, col_id;
+	int col_id, details_value;
 
 	gtk_window_set_title (GTK_WINDOW (editor), _("Bookmarks"));
 
@@ -1232,37 +1370,44 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	g_signal_connect (editor, "delete_event",
 			  G_CALLBACK (delete_event_cb), NULL);
 
-	for (i = 0; i < ephy_bookmark_popup_n_entries; i++)
-	{
-		ephy_bookmark_popup_entries[i].user_data = editor;
-	}
+	editor->priv->main_vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (editor->priv->main_vbox);
+	gtk_container_add (GTK_CONTAINER (editor), editor->priv->main_vbox);
 
-	editor->priv->menu_dock = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (editor->priv->menu_dock);
-	gtk_container_add (GTK_CONTAINER (editor), editor->priv->menu_dock);
-
-	ui_merge = egg_menu_merge_new ();
+	ui_merge = gtk_ui_manager_new ();
 	g_signal_connect (ui_merge, "add_widget", G_CALLBACK (add_widget), editor);
-	action_group = egg_action_group_new ("PopupActions");
-	egg_action_group_add_actions (action_group, ephy_bookmark_popup_entries,
-				      ephy_bookmark_popup_n_entries);
-	egg_menu_merge_insert_action_group (ui_merge,
+	action_group = gtk_action_group_new ("PopupActions");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group, ephy_bookmark_popup_entries,
+				      ephy_bookmark_popup_n_entries, editor);
+	gtk_action_group_add_toggle_actions (action_group,
+					     ephy_bookmark_popup_toggle_entries,
+					     ephy_bookmark_popup_n_toggle_entries,
+					     editor);
+
+	details_value = get_details_value (editor);
+	gtk_action_group_add_radio_actions (action_group,
+					    ephy_bookmark_radio_entries,
+					    ephy_bookmark_n_radio_entries,
+					    details_value,
+					    G_CALLBACK (cmd_view_columns), 
+					    editor);
+	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
-	egg_menu_merge_add_ui_from_file (ui_merge,
+	gtk_ui_manager_add_ui_from_file (ui_merge,
 				         ephy_file ("epiphany-bookmark-editor-ui.xml"),
 				         NULL);
-	gtk_window_add_accel_group (GTK_WINDOW (editor), ui_merge->accel_group);
-	egg_menu_merge_ensure_update (ui_merge);
+	gtk_window_add_accel_group (GTK_WINDOW (editor), 
+				    gtk_ui_manager_get_accel_group (ui_merge));
+	gtk_ui_manager_ensure_update (ui_merge);
+
 	editor->priv->ui_merge = ui_merge;
 	editor->priv->action_group = action_group;
 
-	/* Fixme: We should implement gconf prefs for monitoring this setting */
-	action = egg_action_group_get_action (action_group, "ViewTitle");
-	egg_toggle_action_set_active (EGG_TOGGLE_ACTION (action), TRUE);
-
 	hpaned = gtk_hpaned_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (hpaned), 0);
-	gtk_container_add (GTK_CONTAINER (editor->priv->menu_dock), hpaned);
+	gtk_box_pack_end (GTK_BOX (editor->priv->main_vbox), hpaned,
+			  TRUE, TRUE, 0);
 	gtk_widget_show (hpaned);
 
 	g_assert (editor->priv->bookmarks);
@@ -1358,12 +1503,17 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 					   bmk_drag_types,
 				           n_bmk_drag_types,
 					   col_id);
-	ephy_node_view_add_column (EPHY_NODE_VIEW (bm_view), _("Title"),
+	editor->priv->title_col = ephy_node_view_add_column
+				  (EPHY_NODE_VIEW (bm_view), _("Title"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_TITLE, -1,
 				   EPHY_NODE_VIEW_AUTO_SORT |
 				   EPHY_NODE_VIEW_EDITABLE |
 				   EPHY_NODE_VIEW_SEARCHABLE,
 				   provide_favicon);
+	editor->priv->address_col = ephy_node_view_add_column
+				  (EPHY_NODE_VIEW (bm_view), _("Address"),
+				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_LOCATION,
+				   -1, 0, NULL);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), bm_view);
 	gtk_widget_show (bm_view);
 	editor->priv->bm_view = bm_view;
@@ -1392,6 +1542,8 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	ephy_state_add_paned  (GTK_WIDGET (hpaned),
 			       "bookmarks_paned",
 		               130);
+
+	set_columns_visibility (editor, details_value);
 }
 
 void
@@ -1480,12 +1632,12 @@ toolbar_items_changed_cb (EphyToolbarsModel *model,
 static void
 ephy_bookmarks_editor_init (EphyBookmarksEditor *editor)
 {
-	editor->priv = g_new0 (EphyBookmarksEditorPrivate, 1);
+	editor->priv = EPHY_BOOKMARKS_EDITOR_GET_PRIVATE (editor);
 
 	editor->priv->props_dialogs = g_hash_table_new (g_direct_hash,
                                                         g_direct_equal);
 	editor->priv->tb_model = EPHY_TOOLBARS_MODEL
-		(ephy_shell_get_toolbars_model (ephy_shell));
+		(ephy_shell_get_toolbars_model (ephy_shell, FALSE));
 
 	g_signal_connect (editor->priv->tb_model, "item_added",
 			  G_CALLBACK (toolbar_items_changed_cb), editor);
