@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-import.c,v 1.15.2.2 2003/09/29 18:14:35 chpe Exp $
+ *  $Id: ephy-bookmarks-import.c,v 1.17 2003/11/05 22:56:16 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +28,7 @@
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 
 #include "ephy-bookmarks-import.h"
 #include "ephy-debug.h"
@@ -750,12 +750,7 @@ ephy_bookmarks_import_rdf (EphyBookmarks *bookmarks,
 		return FALSE;
 
 	doc = xmlParseFile (filename);
-	if (doc == NULL)
-	{
-		g_warning ("Failed to re-import bookmarks. You have just lost all your bookmarks!\n");
-		return FALSE;
-	}
-
+	g_assert (doc != NULL);
 	root = xmlDocGetRootElement (doc);
 
 	child = root->children;
