@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-window.h,v 1.19 2003/12/07 10:23:31 chpe Exp $
  */
 
 #ifndef EPHY_WINDOW_H
@@ -29,14 +31,14 @@
 
 G_BEGIN_DECLS
 
+#define EPHY_TYPE_WINDOW	(ephy_window_get_type ())
+#define EPHY_WINDOW(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_WINDOW, EphyWindow))
+#define EPHY_WINDOW_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_WINDOW, EphyWindowClass))
+#define EPHY_IS_WINDOW(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_WINDOW))
+#define EPHY_IS_WINDOW_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_WINDOW))
+#define EPHY_WINDOW_GET_CLASS(o)(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_WINDOW, EphyWindowClass))
+
 typedef struct EphyWindowClass EphyWindowClass;
-
-#define EPHY_WINDOW_TYPE             (ephy_window_get_type ())
-#define EPHY_WINDOW(obj)             (GTK_CHECK_CAST ((obj), EPHY_WINDOW_TYPE, EphyWindow))
-#define EPHY_WINDOW_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), EPHY_WINDOW, EphyWindowClass))
-#define IS_EPHY_WINDOW(obj)          (GTK_CHECK_TYPE ((obj), EPHY_WINDOW_TYPE))
-#define IS_EPHY_WINDOW_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), EPHY_WINDOW))
-
 typedef struct EphyWindow EphyWindow;
 typedef struct EphyWindowPrivate EphyWindowPrivate;
 typedef struct Toolbar Toolbar;
@@ -44,9 +46,11 @@ typedef struct Toolbar Toolbar;
 struct EphyWindow
 {
         GtkWindow parent;
+	/*< private >*/
         EphyWindowPrivate *priv;
 
 	/* Public to toolbar and statusbar, dont use outside */
+	/*< public >*/
 	GObject *ui_merge;
 };
 
@@ -61,11 +65,6 @@ typedef enum
 	FullscreenMode
 } EphyWindowMode;
 
-typedef enum
-{
-	FavoritesControl
-} ControlID;
-
 /* Include the header down here to resolve circular dependency */
 #include "ephy-tab.h"
 
@@ -73,10 +72,11 @@ GType		  ephy_window_get_type		  (void);
 
 EphyWindow	 *ephy_window_new		  (void);
 
-void		  ephy_window_set_chrome	  (EphyWindow *window,
+void		  ephy_window_request_chrome	  (EphyWindow *window,
 						   EmbedChromeMask chrome_flags);
 
-EmbedChromeMask   ephy_window_get_chrome	  (EphyWindow *window);
+void		  ephy_window_set_print_preview	  (EphyWindow *window,
+						   gboolean enabled);
 
 GtkWidget	 *ephy_window_get_notebook	  (EphyWindow *window);
 
@@ -99,9 +99,6 @@ void		  ephy_window_set_zoom		  (EphyWindow *window,
 
 void		  ephy_window_activate_location	  (EphyWindow *window);
 
-void		  ephy_window_update_control	  (EphyWindow *window,
-						   ControlID control);
-
 EphyTab		 *ephy_window_get_active_tab	  (EphyWindow *window);
 
 EphyEmbed	 *ephy_window_get_active_embed	  (EphyWindow *window);
@@ -110,9 +107,7 @@ GList		 *ephy_window_get_tabs		  (EphyWindow *window);
 
 Toolbar		 *ephy_window_get_toolbar	  (EphyWindow *window);
 
-/* Dialogs */
-
-EphyDialog       *ephy_window_get_find_dialog	  (EphyWindow *window);
+void		  ephy_window_find		  (EphyWindow *window);
 
 void		  ephy_window_print		  (EphyWindow *window);
 
