@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,41 +15,46 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.83 2003/10/16 17:28:46 marco Exp $
+ *  $Id: window-commands.c,v 1.100 2003/12/07 10:23:31 chpe Exp $
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "ephy-shell.h"
+#include "ephy-embed-factory.h"
+#include "ephy-embed-persist.h"
 #include "ephy-debug.h"
+#include "ephy-command-manager.h"
 #include "window-commands.h"
 #include "print-dialog.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
-#include "ephy-embed-utils.h"
-#include "pdm-dialog.h"
+#include "ephy-dialog.h"
 #include "ephy-bookmarks-editor.h"
+#include "ephy-history-window.h"
 #include "ephy-new-bookmark.h"
+#include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "toolbar.h"
 #include "ephy-state.h"
 #include "ephy-gui.h"
 #include "ephy-zoom.h"
-#include "prefs-dialog.h"
 #include "ephy-toolbars-model.h"
 #include "egg-editable-toolbar.h"
 #include "egg-toolbar-editor.h"
 
 #include <string.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <bonobo/bonobo-i18n.h>
-#include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomeui/gnome-icon-theme.h>
-#include <libgnome/gnome-program.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <gtk/gtkeditable.h>
 #include <gtk/gtktoggleaction.h>
+#include <libgnomeui/gnome-about.h>
+#include <glib/gi18n.h>
 
 enum
 {
@@ -63,6 +68,35 @@ window_cmd_edit_find (GtkAction *action,
 	ephy_window_find (window);
 }
 
+void
+window_cmd_file_print_setup (GtkAction *action,
+			     EphyWindow *window)
+{
+	EphyDialog *dialog;
+
+	dialog = EPHY_DIALOG (ephy_shell_get_print_setup_dialog (ephy_shell));
+
+	ephy_dialog_show (dialog);
+}
+
+void
+window_cmd_file_print_preview (GtkAction *action,
+			       EphyWindow *window)
+{
+	EphyEmbed *embed;
+	EmbedPrintInfo *info;
+
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (EPHY_IS_EMBED (embed));
+
+	info = ephy_print_get_print_info ();
+	info->preview = TRUE;
+
+	ephy_embed_print (embed, info);
+
+	ephy_window_set_print_preview (window, TRUE);
+}
+
 void
 window_cmd_file_print (GtkAction *action,
 		       EphyWindow *window)
@@ -115,11 +149,12 @@ window_cmd_file_send_to	(GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_get_location (embed, TRUE, &embed_location);
+	embed_location = ephy_embed_get_location (embed, TRUE);
 	location = gnome_vfs_escape_string (embed_location);
 	g_free (embed_location);
 
-	if (ephy_embed_get_title (embed, &title) == G_OK)
+	title = ephy_embed_get_title (embed);
+	if (title != NULL)
 	{
 		char *tmp = gnome_vfs_escape_string (title);
 		g_free (title);
@@ -270,7 +305,12 @@ void
 window_cmd_go_bookmarks (GtkAction *action,
 			 EphyWindow *window)
 {
-	ephy_shell_show_bookmarks_editor (ephy_shell, GTK_WIDGET (window));
+	GtkWidget *bwindow;
+
+	bwindow = ephy_shell_get_bookmarks_editor (ephy_shell);
+	ephy_bookmarks_editor_set_parent (EPHY_BOOKMARKS_EDITOR (bwindow),
+			                  GTK_WIDGET (window));
+	gtk_window_present (GTK_WINDOW (bwindow));
 }
 
 void
@@ -291,9 +331,10 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_get_location (embed, TRUE, &location);
+	location = ephy_embed_get_location (embed, TRUE);
 
-	if (ephy_embed_get_title (embed, &title) != G_OK)
+	title = ephy_embed_get_title (embed);
+	if (title == NULL)
 	{
 		title = g_strdup (_("Untitled"));
 	}
@@ -317,53 +358,43 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	g_free (location);
 }
 
+static void
+open_response_cb (GtkDialog *dialog, gint response, EphyWindow *window)
+{
+	if (response == EPHY_RESPONSE_OPEN)
+	{
+		char *uri, *converted;
+
+		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+		converted = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
+
+		if (converted != NULL)
+		{
+			ephy_window_load_url(window, converted);
+		}
+
+		g_free (converted);
+		g_free (uri);
+        }
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 void
 window_cmd_file_open (GtkAction *action,
 		      EphyWindow *window)
 {
-	char *dir, *ret_dir, *file;
-	EphyEmbedShell *embed_shell;
-	gresult result;
-	EphyEmbedSingle *single;
+	EphyFileChooser *dialog;
 
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
+	dialog = ephy_file_chooser_new (_("Open"),
+					GTK_WIDGET (window),
+					GTK_FILE_CHOOSER_ACTION_OPEN,
+					CONF_STATE_OPEN_DIR);
 
-	embed_shell = EPHY_EMBED_SHELL (ephy_shell);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (open_response_cb), window);
 
-        dir = eel_gconf_get_string (CONF_STATE_OPEN_DIR);
-
-	result = ephy_embed_single_show_file_picker
-		(single, GTK_WIDGET (window),
-		 _("Open"),
-                 dir, NULL, modeOpen,
-                 &file, NULL, NULL);
-
-	/* persist directory choice */
-	/* Fix for bug 122780:
-	 * if the user selected a directory, or aborted with no filename typed,
-	 * g_path_get_dirname and gnome_vfs_uri_extract_dirname strip the last
-	 * path component, so test if the returned file is actually a directory.
-	 */
-	if (g_file_test (file, G_FILE_TEST_IS_DIR))
-	{
-		ret_dir = g_strdup (file);
-	}
-	else
-	{
-		ret_dir = g_path_get_dirname (file);
-	}
-
-	eel_gconf_set_string (CONF_STATE_OPEN_DIR, ret_dir);
-
-	if (result == G_OK)
-	{
-		ephy_window_load_url(window, file);
-        }
-
-	g_free (ret_dir);
-	g_free (file);
-        g_free (dir);
+	gtk_widget_show (GTK_WIDGET (dialog));
 }
 
 void
@@ -376,14 +407,18 @@ window_cmd_file_save_as (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	persist = ephy_embed_persist_new (embed);
-	ephy_embed_persist_set_flags (persist,
-				      EMBED_PERSIST_MAINDOC);
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
 
-	ephy_embed_utils_save (GTK_WIDGET(window),
-			       _("Save As"),
-			       CONF_STATE_SAVE_DIR,
-			       TRUE, persist);
+	ephy_embed_persist_set_embed (persist, embed);
+	ephy_embed_persist_set_fc_title (persist, _("Save As"));
+	ephy_embed_persist_set_fc_parent (persist,GTK_WINDOW (window));
+	ephy_embed_persist_set_flags
+		(persist, EMBED_PERSIST_MAINDOC | EMBED_PERSIST_ASK_DESTINATION);
+	ephy_embed_persist_set_persist_key
+		(persist, CONF_STATE_SAVE_DIR);
+
+	ephy_embed_persist_save (persist);
 
 	g_object_unref (G_OBJECT(persist));
 }
@@ -400,6 +435,40 @@ window_cmd_file_close_window (GtkAction *action,
 	ephy_window_remove_tab (window, tab);
 }
 
+void
+window_cmd_edit_undo (GtkAction *action,
+		      EphyWindow *window)
+{
+	GtkWidget *widget;
+	GtkWidget *embed;
+
+	widget = gtk_window_get_focus (GTK_WINDOW (window));
+	embed = gtk_widget_get_ancestor (widget, EPHY_TYPE_EMBED);
+
+	if (embed)
+	{
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
+						 "cmd_undo");
+	}
+}
+
+void
+window_cmd_edit_redo (GtkAction *action,
+		      EphyWindow *window)
+{
+	GtkWidget *widget;
+	GtkWidget *embed;
+
+	widget = gtk_window_get_focus (GTK_WINDOW (window));
+	embed = gtk_widget_get_ancestor (widget, EPHY_TYPE_EMBED);
+
+	if (embed)
+	{
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
+						 "cmd_redo");
+	}
+}
+
 void
 window_cmd_edit_cut (GtkAction *action,
 		     EphyWindow *window)
@@ -416,7 +485,8 @@ window_cmd_edit_cut (GtkAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_selection_cut (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_cut");
 	}
 }
 
@@ -437,7 +507,8 @@ window_cmd_edit_copy (GtkAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_selection_copy (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_copy");
 	}
 }
 
@@ -458,7 +529,8 @@ window_cmd_edit_paste (GtkAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_paste (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_paste");
 	}
 }
 
@@ -479,7 +551,8 @@ window_cmd_edit_select_all (GtkAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_select_all (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_selectAll");
 	}
 }
 
@@ -511,57 +584,30 @@ void
 window_cmd_view_bookmarks_bar (GtkAction *action,
 			       EphyWindow *window)
 {
-	EmbedChromeMask mask;
 	gboolean active;
-	gboolean current_state;
 
-	mask = ephy_window_get_chrome (window);
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	current_state = (mask & EMBED_CHROME_BOOKMARKSBARON) > 0;
-
-	if (active != current_state)
-	{
-		mask ^= EMBED_CHROME_BOOKMARKSBARON;
-		ephy_window_set_chrome (window, mask);
-	}
+	eel_gconf_set_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR, active);
 }
 
 void
 window_cmd_view_toolbar (GtkAction *action,
 			 EphyWindow *window)
 {
-	EmbedChromeMask mask;
 	gboolean active;
-	gboolean current_state;
 
-	mask = ephy_window_get_chrome (window);
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	current_state = (mask & EMBED_CHROME_TOOLBARON) > 0;
-
-	if (active != current_state)
-	{
-		mask ^= EMBED_CHROME_TOOLBARON;
-		ephy_window_set_chrome (window, mask);
-	}
+	eel_gconf_set_boolean (CONF_WINDOWS_SHOW_TOOLBARS, active);
 }
 
 void
 window_cmd_view_statusbar (GtkAction *action,
 			   EphyWindow *window)
 {
-	EmbedChromeMask mask;
 	gboolean active;
-	gboolean current_state;
 
-	mask = ephy_window_get_chrome (window);
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	current_state = (mask & EMBED_CHROME_STATUSBARON) > 0;
-
-	if (active != current_state)
-	{
-		mask ^= EMBED_CHROME_STATUSBARON;
-		ephy_window_set_chrome (window, mask);
-	}
+	eel_gconf_set_boolean (CONF_WINDOWS_SHOW_STATUSBAR, active);
 }
 
 void
@@ -599,25 +645,150 @@ window_cmd_view_zoom_normal (GtkAction *action,
 	ephy_window_set_zoom (window, 1.0);
 }
 
+static GnomeVFSMimeApplication *
+get_editor_application (void)
+{
+	GnomeVFSMimeApplication *app;
+
+	app = gnome_vfs_mime_get_default_application ("text/plain");
+	if (app == NULL)
+	{
+		g_warning ("Cannot find a text editor.");
+	}
+	return app;
+}
+
+static void
+editor_open_uri (const char *address)
+{
+	GList *uris = NULL;
+	char *canonical;
+	GnomeVFSMimeApplication *app;
+
+	canonical = gnome_vfs_make_uri_canonical (address);
+
+	uris = g_list_append (uris, canonical);
+
+	app = get_editor_application ();
+	if (app)
+	{
+		gnome_vfs_mime_application_launch (app, uris);
+		gnome_vfs_mime_application_free (app);
+	}
+
+	g_free (canonical);
+	g_list_free (uris);
+}
+
+static void
+save_source_completed_cb (EphyEmbedPersist *persist)
+{
+	const char *dest;
+
+	dest = ephy_embed_persist_get_dest (persist);
+	g_return_if_fail (dest != NULL);
+
+	ephy_shell_delete_on_exit (ephy_shell, dest);
+
+	editor_open_uri (dest);
+}
+
+static gboolean
+editor_can_open_uri (char *address)
+{
+	GnomeVFSMimeApplication *app;
+	GnomeVFSURI *uri;
+	const char *scheme;
+	gboolean result = FALSE;
+
+	app = get_editor_application ();
+
+	uri = gnome_vfs_uri_new (address);
+	scheme = uri ? gnome_vfs_uri_get_scheme (uri) : NULL;
+
+	/* Open directly only read/write protocols, otherwise
+	   you just get extra network overhead without any advantage */
+	if (scheme && strcmp (scheme, "file") != 0)
+	{
+		scheme = NULL;
+	}
+	
+	if (scheme && app && app->supported_uri_schemes)
+	{
+		if (g_list_find_custom (app->supported_uri_schemes,
+                                        scheme, (GCompareFunc) strcmp))
+		{
+			result = TRUE;
+		}
+	}
+
+	if (uri)
+	{
+		gnome_vfs_uri_unref (uri);
+	}
+
+	return result;
+}
+
+static void
+save_temp_source (EphyEmbed *embed)
+{
+	char *tmp, *base;
+	EphyEmbedPersist *persist;
+
+	base = g_build_filename (g_get_tmp_dir (), "viewsourceXXXXXX", NULL);
+	tmp = ephy_file_tmp_filename (base, "html");
+	g_free (base);
+
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+
+	ephy_embed_persist_set_embed (persist, embed);
+	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_COPY_PAGE |
+				      EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_dest (persist, tmp);
+
+	g_signal_connect (persist, "completed",
+			  G_CALLBACK (save_source_completed_cb), NULL);
+
+	ephy_embed_persist_save (persist);
+	g_object_unref (persist);
+
+	g_free (tmp);
+}
+
 void
 window_cmd_view_page_source (GtkAction *action,
 			     EphyWindow *window)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
+	char *address;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	ephy_shell_new_tab (ephy_shell, window, tab, NULL,
-			    EPHY_NEW_TAB_CLONE_PAGE |
-			    EPHY_NEW_TAB_SOURCE_MODE);
+	address = ephy_embed_get_location (embed, TRUE);
+
+	if (editor_can_open_uri (address))
+	{
+		editor_open_uri (address);
+	}
+	else
+	{
+		save_temp_source (embed);
+	}
 }
 
 void
 window_cmd_go_history (GtkAction *action,
 		       EphyWindow *window)
 {
-	ephy_shell_show_history_window (ephy_shell, GTK_WIDGET (window));
+	GtkWidget *hwindow;
+
+	hwindow = ephy_shell_get_history_window (ephy_shell);
+	ephy_history_window_set_parent (EPHY_HISTORY_WINDOW (hwindow),
+					GTK_WIDGET (window));
+	gtk_window_present (GTK_WINDOW (hwindow));
 }
 
 void
@@ -626,7 +797,7 @@ window_cmd_edit_personal_data (GtkAction *action,
 {
 	EphyDialog *dialog;
 
-	dialog = pdm_dialog_new (GTK_WIDGET(window));
+	dialog = EPHY_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell));
 
 	ephy_dialog_show (dialog);
 }
@@ -637,7 +808,8 @@ window_cmd_edit_prefs (GtkAction *action,
 {
 	EphyDialog *dialog;
 
-	dialog = prefs_dialog_new (GTK_WIDGET (window));
+	dialog = EPHY_DIALOG (ephy_shell_get_prefs_dialog (ephy_shell));
+
 	ephy_dialog_show (dialog);
 }
 
@@ -669,7 +841,7 @@ toolbar_editor_response_cb (GtkDialog  *dialog,
 						n - 1, "UserCreated");
 		break;
 	case GTK_RESPONSE_HELP:
-		/* FIXME: Connect toolbar editor help */
+		ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "to-edit-toolbars");
 		break;
 	}
 }
@@ -742,26 +914,27 @@ window_cmd_help_about (GtkAction *action,
 	static GtkWidget *about = NULL;
 	GtkWidget** ptr;
 	GdkPixbuf *icon;
-	char *icon_path;
-	GdkPixbuf *logo;
-	GnomeIconTheme *icon_theme;
+	const char *icon_path;
+	GdkPixbuf *logo = NULL;
+	GtkIconTheme *icon_theme;
+	GtkIconInfo *icon_info;
 
-	static gchar *authors[] = {
+	static char *authors[] = {
 		"Marco Pesenti Gritti <marco@gnome.org>",
 		"Xan Lopez <xan@masilla.org>",
 		"David Bordoley <bordoley@msu.edu>",
-		"Christian Persch <chpe+epiphany@stud.uni-saarland.de>",
+		"Christian Persch <chpe@gnome.org>",
 		NULL
 	};
 
-	gchar *documenters[] = {
+	char *documenters[] = {
 		"Patanjali Somayaji <patanjali@codito.com>",
 		"David Bordoley <bordoley@msu.edu>",
 		NULL
 	};
 
 	/* Translator credits */
-	gchar *translator_credits = _("translator_credits");
+	char *translator_credits = _("translator_credits");
 
 	if (about != NULL)
 	{
@@ -770,19 +943,21 @@ window_cmd_help_about (GtkAction *action,
 		return;
 	}
 
-	icon_theme = gnome_icon_theme_new ();
-	icon_path = gnome_icon_theme_lookup_icon (icon_theme, "web-browser",
-						  -1, NULL, NULL);
-	g_object_unref (icon_theme);
+	/* FIXME: use the icon theme for the correct screen, not for the default screen */
+	icon_theme = gtk_icon_theme_get_default ();
+	icon_info = gtk_icon_theme_lookup_icon (icon_theme, "web-browser", -1, 0);
 
-	if (icon_path)
+	if (icon_info)
 	{
-		logo = gdk_pixbuf_new_from_file (icon_path, NULL);
-		g_free (icon_path);
+		icon_path = gtk_icon_info_get_filename (icon_info);
+
+		if (icon_path != NULL)
+		{
+			logo = gdk_pixbuf_new_from_file (icon_path, NULL);
+		}
 	}
 	else
 	{
-		logo = NULL;
 		g_warning ("Web browser gnome icon not found");
 	}
 
@@ -795,7 +970,10 @@ window_cmd_help_about (GtkAction *action,
 		       strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 		       logo);
 
-	g_object_unref (logo);
+	if (logo != NULL)
+	{
+		g_object_unref (logo);
+	}
 
 	gtk_window_set_transient_for (GTK_WINDOW (about),
 				      GTK_WINDOW (window));
